@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2022, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -22,10 +22,12 @@
  */
 
 #include "modules/adminapi/common/cluster_topology_executor.h"
+#include "modules/adminapi/common/connectivity_check.h"
 #include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/instance_validations.h"
 #include "modules/adminapi/common/member_recovery_monitoring.h"
 #include "modules/adminapi/common/provision.h"
+#include "modules/adminapi/common/server_features.h"
 #include "modules/adminapi/common/validations.h"
 #include "mysqlshdk/libs/mysql/async_replication.h"
 #include "mysqlshdk/libs/mysql/clone.h"
@@ -62,7 +64,8 @@ void Add_instance::resolve_local_address(
     Group_replication_options *gr_options,
     const Group_replication_options &user_gr_options) {
   auto hostname = m_target_instance->get_canonical_hostname();
-  auto communication_stack = m_cluster_impl->get_communication_stack();
+  auto communication_stack =
+      get_communication_stack(*m_cluster_impl->get_cluster_server());
 
   gr_options->local_address = mysqlsh::dba::resolve_gr_local_address(
       user_gr_options.local_address.value_or("").empty()
@@ -227,13 +230,15 @@ void Add_instance::handle_clone_plugin_state(bool enable_clone) const {
 bool Add_instance::create_replication_user() {
   // Creates the replication user ONLY if not already given.
   // NOTE: User always created at the seed instance.
-  if (!m_options.gr_options.recovery_credentials ||
-      m_options.gr_options.recovery_credentials->user.empty()) {
-    std::tie(m_options.gr_options.recovery_credentials, m_account_host) =
-        m_cluster_impl->create_replication_user(m_target_instance.get());
-    return true;
-  }
-  return false;
+  if (m_options.gr_options.recovery_credentials &&
+      !m_options.gr_options.recovery_credentials->user.empty())
+    return false;
+
+  std::tie(m_options.gr_options.recovery_credentials, m_account_host) =
+      m_cluster_impl->create_replication_user(m_target_instance.get(),
+                                              m_options.cert_subject);
+
+  return true;
 }
 
 void Add_instance::clean_replication_user() {
@@ -290,35 +295,18 @@ void Add_instance::wait_recovery(const std::string &join_begin_time,
 
 void Add_instance::refresh_target_connections() const {
   try {
-    m_target_instance->query("SELECT 1");
+    m_target_instance->reconnect_if_needed("Target");
   } catch (const shcore::Error &err) {
-    auto e = shcore::Exception::mysql_error_with_code(err.what(), err.code());
+    auto cluster_coptions =
+        m_cluster_impl->get_cluster_server()->get_connection_options();
 
-    if (mysqlshdk::db::is_mysql_client_error(e.code())) {
-      log_debug(
-          "Connection to instance '%s' lost: %s. Re-establishing a "
-          "connection.",
-          m_target_instance->get_canonical_address().c_str(),
-          e.format().c_str());
-
-      try {
-        m_target_instance->get_session()->connect(
-            m_target_instance->get_connection_options());
-      } catch (const shcore::Error &ie) {
-        auto cluster_coptions =
-            m_cluster_impl->get_cluster_server()->get_connection_options();
-
-        if (ie.code() == ER_ACCESS_DENIED_ERROR &&
-            m_target_instance->get_connection_options().get_user() !=
-                cluster_coptions.get_user()) {
-          // try again with cluster credentials
-          auto copy = m_target_instance->get_connection_options();
-          copy.set_login_options_from(cluster_coptions);
-          m_target_instance->get_session()->connect(copy);
-        } else {
-          throw;
-        }
-      }
+    if (err.code() == ER_ACCESS_DENIED_ERROR &&
+        m_target_instance->get_connection_options().get_user() !=
+            cluster_coptions.get_user()) {
+      // try again with cluster credentials
+      auto copy = m_target_instance->get_connection_options();
+      copy.set_login_options_from(cluster_coptions);
+      m_target_instance->get_session()->connect(copy);
       m_target_instance->prepare_session();
     } else {
       throw;
@@ -357,7 +345,7 @@ void Add_instance::store_local_replication_account() const {
                     m_account_host);
           } catch (const std::exception &e) {
             log_info(
-                "Failed updating Metatada of '%s' to include the recovery "
+                "Failed updating Metadata of '%s' to include the recovery "
                 "account of the joining member",
                 instance->descr().c_str());
           }
@@ -438,10 +426,13 @@ void Add_instance::restore_group_replication_account() const {
       m_target_instance->get_version());
   log_debug("Re-issuing the CHANGE %s command", source_term.c_str());
 
+  mysqlshdk::mysql::Replication_credentials_options options;
+  options.password =
+      m_options.gr_options.recovery_credentials->password.value_or("");
+
   mysqlshdk::mysql::change_replication_credentials(
-      *m_target_instance, m_options.gr_options.recovery_credentials->user,
-      m_options.gr_options.recovery_credentials->password.value_or(""),
-      mysqlshdk::gr::k_gr_recovery_channel);
+      *m_target_instance, mysqlshdk::gr::k_gr_recovery_channel,
+      m_options.gr_options.recovery_credentials->user, options);
 
   // Update the recovery account to the right one
   log_debug("Adding recovery account to metadata");
@@ -474,7 +465,7 @@ void Add_instance::prepare() {
 
   // Validate the options used
   cluster_topology_executor_ops::validate_add_rejoin_options(
-      m_options.gr_options, m_cluster_impl->get_communication_stack());
+      m_options.gr_options, get_communication_stack(*m_primary_instance));
 
   // Make sure there isn't some leftover auto-rejoin active
   cluster_topology_executor_ops::is_member_auto_rejoining(m_target_instance);
@@ -501,7 +492,7 @@ void Add_instance::prepare() {
 
   // Verify whether the instance supports the communication stack in use in
   // the Cluster and set it in the options handler
-  m_comm_stack = m_cluster_impl->get_communication_stack();
+  m_comm_stack = get_communication_stack(*m_primary_instance);
 
   cluster_topology_executor_ops::check_comm_stack_support(
       m_target_instance, &m_options.gr_options, m_comm_stack);
@@ -522,9 +513,52 @@ void Add_instance::prepare() {
         m_target_instance->get_canonical_port());
   }
 
+  // recovery auth checks
+  {
+    auto auth_type = m_cluster_impl->query_cluster_auth_type();
+
+    // check if member auth request mode is supported
+    validate_instance_member_auth_type(*m_target_instance, false,
+                                       m_options.gr_options.ssl_mode,
+                                       "memberSslMode", auth_type);
+
+    // check if certSubject was correctly supplied
+    validate_instance_member_auth_options("cluster", false, auth_type,
+                                          m_options.cert_subject);
+
+    // Check connectivity and SSL
+    if (current_shell_options()->get().dba_connectivity_checks) {
+      current_console()->print_info(
+          "* Checking connectivity and SSL configuration...");
+
+      std::string cert_issuer =
+          m_cluster_impl->query_cluster_auth_cert_issuer();
+      mysqlshdk::mysql::Set_variable disable_sro(
+          *m_target_instance, "super_read_only", false, true);
+      test_peer_connection(
+          *m_target_instance, m_options.gr_options.local_address.value_or(""),
+          m_options.cert_subject, *m_primary_instance,
+          m_primary_instance
+              ->get_sysvar_string("group_replication_local_address")
+              .value_or(""),
+          m_cluster_impl->query_cluster_instance_auth_cert_subject(
+              m_primary_instance->get_uuid()),
+          m_options.gr_options.ssl_mode, auth_type, cert_issuer, m_comm_stack);
+    }
+  }
+
   // Set the transaction size limit
   m_options.gr_options.transaction_size_limit =
-      m_cluster_impl->get_transaction_size_limit();
+      get_transaction_size_limit(*m_primary_instance);
+
+  // Set the paxos_single_leader option
+  // Verify whether the primary supports it, meaning it's in use. Checking if
+  // the target supports it is not valid since the target might be 8.0 and the
+  // primary 5.7
+  if (supports_paxos_single_leader(m_primary_instance->get_version())) {
+    m_options.gr_options.paxos_single_leader =
+        get_paxos_single_leader_enabled(*m_primary_instance).value_or(false);
+  }
 }
 
 void Add_instance::do_run() {
@@ -591,16 +625,12 @@ void Add_instance::do_run() {
           m_cluster_impl->get_id());
 
   // Clone handling
-  bool clone_disabled = m_cluster_impl->get_disable_clone_option();
-  bool clone_supported =
-      mysqlshdk::mysql::is_clone_available(*m_target_instance);
-
   int64_t restore_clone_threshold = 0;
   bool restore_recovery_accounts = false;
-
-  if (clone_supported) {
+  if (mysqlshdk::mysql::is_clone_available(*m_target_instance)) {
     // Clone must be uninstalled only when disableClone is enabled on the
     // Cluster
+    bool clone_disabled = m_cluster_impl->get_disable_clone_option();
     bool enable_clone = !clone_disabled;
 
     // Force clone if requested
@@ -688,6 +718,20 @@ void Add_instance::do_run() {
     std::string join_begin_time =
         m_target_instance->queryf_one_string(0, "", "SELECT NOW(3)");
 
+    bool recovery_certificates{false};
+    bool recovery_needs_password{true};
+    switch (m_cluster_impl->query_cluster_auth_type()) {
+      case mysqlsh::dba::Replication_auth_type::CERT_ISSUER:
+      case mysqlsh::dba::Replication_auth_type::CERT_SUBJECT:
+        recovery_needs_password = false;
+        [[fallthrough]];
+      case mysqlsh::dba::Replication_auth_type::CERT_ISSUER_PASSWORD:
+      case mysqlsh::dba::Replication_auth_type::CERT_SUBJECT_PASSWORD:
+        recovery_certificates = true;
+      default:
+        break;
+    }
+
     {
       if (m_already_member) {
         // START GROUP_REPLICATION is the last thing that happens in
@@ -697,7 +741,7 @@ void Add_instance::do_run() {
                  m_target_instance->descr().c_str());
       } else {
         log_info(
-            "Joining '%s' to cluster using account %s to peer '%s'.",
+            "Joining '%s' to cluster using account '%s' to peer '%s'.",
             m_target_instance->descr().c_str(),
             m_primary_instance->get_connection_options().get_user().c_str(),
             m_primary_instance->descr().c_str());
@@ -709,25 +753,55 @@ void Add_instance::do_run() {
         // disabled before starting GR, otherwise we'd create errant
         // transaction
         if (m_comm_stack == kCommunicationStackMySQL) {
-          m_cluster_impl->create_local_replication_user(m_target_instance,
-                                                        m_options.gr_options);
+          // NOTE: if the recovery users use certificates, we can't change GR's
+          // recovery replication credentials in all possible donors. That would
+          // simply fail, because in any given donor other than the new
+          // instance, using the recovery user of the new instance will fail
+          // (because the certificate only exists on the new instance server).
+          m_cluster_impl->create_local_replication_user(
+              m_target_instance, m_options.cert_subject, m_options.gr_options,
+              !recovery_certificates);
+
+          // if recovery accounts need certificates, we much ensure that the
+          // recovery accounts of all members also exist on the target instance
+          if (recovery_certificates) {
+            m_cluster_impl->create_replication_users_at_instance(
+                m_target_instance);
+          }
 
           // At this point, all Cluster members got the recovery credentials
-          // changed to be able to use the account created for the joining
-          // member so in case of failure, we must restore all
-          restore_recovery_accounts = true;
+          // changed (if certificates aren't needed), to be able to use the
+          // account created for the joining member so in case of failure, we
+          // must restore all
+          restore_recovery_accounts = !recovery_certificates;
 
           DBUG_EXECUTE_IF("fail_add_instance_mysql_stack", {
             // Throw an exception to simulate a failure at
             // this point in addInstance()
             throw std::logic_error("debug");
           });
+        } else if (recovery_certificates && (restore_clone_threshold > 0)) {
+          // reaching this point, we're using 'XCOM' communication stack, the
+          // recovery users use certificates and the recovery process will use
+          // clone. This means that, at the end of the clone, GR in the new
+          // instance will use the recovery user from the seed instance, which
+          // will fail because of the certificates. To solve this, we kind of
+          // imitate the behavior of the MySQL comm stack: we create the new
+          // replication user on all existing members, change the recovery
+          // credentials on them to use the new user (so that the clone leaves
+          // the new instance configured properly) and then revert everything
+          // back
+          m_cluster_impl->create_local_replication_user(
+              m_target_instance, m_options.cert_subject, m_options.gr_options,
+              true);
+
+          restore_recovery_accounts = true;
         }
 
         // Join the instance to the Group Replication group.
         mysqlsh::dba::join_cluster(*m_target_instance, *m_primary_instance,
-                                   m_options.gr_options, cluster_member_count,
-                                   cfg.get());
+                                   m_options.gr_options, recovery_certificates,
+                                   cluster_member_count, cfg.get());
       }
 
       // Wait until recovery done. Will throw an exception if recovery fails.
@@ -758,11 +832,15 @@ void Add_instance::do_run() {
     // If the instance is not on the Metadata, we must add it.
     if (!is_instance_on_md) {
       m_cluster_impl->add_metadata_for_instance(*m_target_instance,
-                                                m_options.label.get_safe());
+                                                m_options.label.value_or(""));
       m_cluster_impl->get_metadata_storage()->update_instance_attribute(
           m_target_instance->get_uuid(), k_instance_attribute_join_time,
           shcore::Value(join_begin_time));
 
+      m_cluster_impl->get_metadata_storage()->update_instance_attribute(
+          m_target_instance->get_uuid(), k_instance_attribute_cert_subject,
+          shcore::Value(m_options.cert_subject));
+
       // Store the username in the Metadata instances table
       if (owns_repl_user) {
         if (m_comm_stack == kCommunicationStackMySQL &&
@@ -775,7 +853,7 @@ void Add_instance::do_run() {
     }
 
     // Update group_seeds and auto_increment in other members
-    m_cluster_impl->update_group_peers(m_target_instance, m_options.gr_options,
+    m_cluster_impl->update_group_peers(*m_target_instance, m_options.gr_options,
                                        cluster_member_count,
                                        address_in_metadata);
 
@@ -826,9 +904,22 @@ void Add_instance::do_run() {
     // finished so the credentials for the replication channel (that are
     // cloned to the instance joining) won't match since they were already
     // reset in the Cluster by restore_recovery_account_all_members().
-    if (m_comm_stack == kCommunicationStackMySQL &&
-        m_progress_style != Recovery_progress_style::NOWAIT) {
-      m_cluster_impl->restore_recovery_account_all_members();
+    //
+    // Another scenario to take into account (also detailed above), where in
+    // case of XCOM comm stack, the recovery users needing certificates and
+    // using clone, all members are currently using the new recovery user, which
+    // means that we need to restore them all back to their corresponding users.
+    {
+      auto restore_accounts =
+          (m_comm_stack == kCommunicationStackMySQL) &&
+          !recovery_certificates &&
+          (m_progress_style != Recovery_progress_style::NOWAIT);
+      restore_accounts |= (m_comm_stack != kCommunicationStackMySQL) &&
+                          recovery_certificates &&
+                          (restore_clone_threshold > 0);
+      if (restore_accounts)
+        m_cluster_impl->restore_recovery_account_all_members(
+            recovery_needs_password);
     }
 
     // Only commit transaction once everything is done, so that things don't
