@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -162,21 +162,41 @@ void Rescan::prepare() {
   auto console = mysqlsh::current_console();
 
   // All instance to add must be an active member of the GR group.
-  std::vector<std::string> invalid_add_members =
-      detect_invalid_members(m_options.add_instances_list, true);
+  {
+    std::vector<std::string> invalid_add_members =
+        detect_invalid_members(m_options.add_instances_list, true);
 
-  if (!invalid_add_members.empty()) {
+    if (!invalid_add_members.empty()) {
+      console->print_error(
+          "The following instances cannot be added because they are not active "
+          "members of the cluster: '" +
+          shcore::str_join(invalid_add_members, ", ") +
+          "'. Please verify if the specified addresses are correct, or if the "
+          "instances are currently inactive.");
+      console->print_info();
+
+      throw shcore::Exception::runtime_error(
+          "The following instances are not active members of the cluster: '" +
+          shcore::str_join(invalid_add_members, ", ") + "'.");
+    }
+  }
+
+  // can't add instances if recovery auth isn't password
+  if (((!m_options.add_instances_list.empty() || m_options.auto_add) &&
+       (m_cluster->query_cluster_auth_type() !=
+        Replication_auth_type::PASSWORD))) {
     console->print_error(
-        "The following instances cannot be added because they are not active "
-        "members of the cluster: '" +
-        shcore::str_join(invalid_add_members, ", ") +
-        "'. Please verify if the specified addresses are correct, or if the "
-        "instances are currently inactive.");
+        "Unrecognized members were detected in the group, but the cluster is "
+        "configured to require SSL certificate authentication. Please stop GR "
+        "on those members and then add them to the cluster using "
+        "cluster.<<<addInstance>>>() with the appropriate authentication "
+        "options.");
     console->print_info();
 
-    throw shcore::Exception::runtime_error(
-        "The following instances are not active members of the cluster: '" +
-        shcore::str_join(invalid_add_members, ", ") + "'.");
+    throw shcore::Exception::runtime_error(shcore::str_format(
+        "Can't automatically add unrecognized members to the cluster when "
+        "memberAuthType is '%s'.",
+        to_string(m_cluster->query_cluster_auth_type()).c_str()));
   }
 
   // All instance to remove cannot be an active member of the GR group.
@@ -197,8 +217,8 @@ void Rescan::prepare() {
   }
 
   // Check if group_replication_view_change_uuid is supported
-  m_is_view_change_uuid_supported =
-      m_cluster->get_lowest_instance_version() >= k_min_cs_version;
+  m_is_view_change_uuid_supported = m_cluster->get_lowest_instance_version() >=
+                                    Precondition_checker::k_min_cs_version;
 
   // Validate the usage of the option 'updateViewChangeUuid'
   if (m_options.update_view_change_uuid.get_safe() &&
@@ -501,8 +521,19 @@ Instance_list::const_iterator find_in_instance_list(
 
 void Rescan::add_metadata_for_instances(
     const std::shared_ptr<shcore::Value::Array_type> &instance_list) {
+  if (instance_list->empty()) return;
+
   auto console = mysqlsh::current_console();
 
+  if (m_cluster->query_cluster_auth_type() != Replication_auth_type::PASSWORD) {
+    console->print_info(
+        "New instances were discovered in the cluster but ignore because the "
+        "cluster requires SSL certificate authentication.\nPlease stop GR on "
+        "those members and then add them to the cluster using "
+        "cluster.addInstance() with the appropriate authentication options.");
+    return;
+  }
+
   for (const auto &instance : *instance_list) {
     auto instance_map = instance.as_map();
 
@@ -648,10 +679,6 @@ void Rescan::upgrade_comm_protocol() {
 }
 
 void Rescan::ensure_view_change_uuid_set() {
-  auto console = mysqlsh::current_console();
-  std::shared_ptr<Instance> cluster_instance = m_cluster->get_cluster_server();
-  std::string view_change_uuid;
-
   // Only generate and set the value when all Cluster members are running a
   // version >= k_min_cs_version
   // NOTE: group_replication_view_change_uuid must be the same on all Cluster
@@ -659,55 +686,65 @@ void Rescan::ensure_view_change_uuid_set() {
   // know about the sysvar (<= 8.0.26) will be considered as having the sysvar
   // set to AUTOMATIC so we must ensure the sysvar is only set when all members
   // support it
-  if (m_cluster->get_lowest_instance_version() >= k_min_cs_version) {
-    // Check if group_replication_view_change_uuid is already set on the cluster
-    view_change_uuid = cluster_instance->get_sysvar_string(
-        "group_replication_view_change_uuid", "");
+  if (m_cluster->get_lowest_instance_version() <
+      Precondition_checker::k_min_cs_version)
+    return;
 
-    // If not set (AUTOMATIC), generate a new value and set it on all members
-    if (view_change_uuid == "AUTOMATIC") {
-      // Check if the variable was persisted so it needs a restart
-      std::string view_change_uuid_persisted =
-          cluster_instance
-              ->get_persisted_value("group_replication_view_change_uuid")
-              .value_or("");
+  auto console = mysqlsh::current_console();
 
-      if (!view_change_uuid_persisted.empty() &&
-          view_change_uuid_persisted != view_change_uuid) {
-        console->print_note(
-            "The Cluster's group_replication_view_change_uuid is set but not "
-            "yet effective");
-      } else {
-        console->print_note(
-            "The Cluster's group_replication_view_change_uuid is not set");
-        console->print_info(
-            "Generating and setting a value for "
-            "group_replication_view_change_uuid...");
+  // Check if group_replication_view_change_uuid is already set on the cluster
+  std::shared_ptr<Instance> cluster_instance = m_cluster->get_cluster_server();
+  auto view_change_uuid = cluster_instance->get_sysvar_string(
+      "group_replication_view_change_uuid", "");
 
-        // Generate a value for group_replication_view_change_uuid
-        view_change_uuid = mysqlshdk::gr::generate_uuid(*cluster_instance);
+  bool skip_md_mismatch_check = false;
 
-        // Get the Cluster Config Object
-        // This ensures the config is set on ONLINE || RECOVERING members
-        auto cfg = m_cluster->create_config_object({}, false, true);
+  // If not set (AUTOMATIC), generate a new value and set it on all members
+  if (view_change_uuid == "AUTOMATIC") {
+    // Check if the variable was persisted so it needs a restart
+    std::string view_change_uuid_persisted =
+        cluster_instance
+            ->get_persisted_value("group_replication_view_change_uuid")
+            .value_or("");
 
-        cfg->set("group_replication_view_change_uuid", view_change_uuid);
+    if (!view_change_uuid_persisted.empty() &&
+        view_change_uuid_persisted != view_change_uuid) {
+      console->print_note(
+          "The Cluster's group_replication_view_change_uuid is set but not "
+          "yet effective");
 
-        cfg->apply();
-      }
+      skip_md_mismatch_check = true;
+    } else {
+      console->print_note(
+          "The Cluster's group_replication_view_change_uuid is not set");
+      console->print_info(
+          "Generating and setting a value for "
+          "group_replication_view_change_uuid...");
 
-      console->print_warning(
-          "The Cluster must be completely taken OFFLINE and restarted "
-          "(<<<dba.rebootClusterFromCompleteOutage>>>()) for the settings to "
-          "be effective");
+      // Generate a value for group_replication_view_change_uuid
+      view_change_uuid = mysqlshdk::gr::generate_uuid(*cluster_instance);
+
+      // Get the Cluster Config Object
+      // This ensures the config is set on ONLINE || RECOVERING members
+      auto cfg = m_cluster->create_config_object({}, false, true);
+
+      cfg->set("group_replication_view_change_uuid", view_change_uuid);
+
+      cfg->apply();
     }
 
-    ensure_view_change_uuid_set_stored_metadata(view_change_uuid);
+    console->print_warning(
+        "The Cluster must be completely taken OFFLINE and restarted "
+        "(<<<dba.rebootClusterFromCompleteOutage>>>()) for the settings to "
+        "be effective");
   }
+
+  ensure_view_change_uuid_set_stored_metadata(view_change_uuid,
+                                              skip_md_mismatch_check);
 }
 
 void Rescan::ensure_view_change_uuid_set_stored_metadata(
-    const std::string &view_change_uuid) {
+    const std::string &view_change_uuid, bool skip_md_mismatch_check) {
   auto console = mysqlsh::current_console();
 
   // Check if view_change_uuid is stored in the Metadata
@@ -726,6 +763,24 @@ void Rescan::ensure_view_change_uuid_set_stored_metadata(
           m_cluster->get_id(), "group_replication_view_change_uuid",
           shcore::Value(view_change_uuid));
     }
+  } else if (!skip_md_mismatch_check) {
+    // Ensure that the value of group_replication_view_change_uuid set on the
+    // cluster matches the value stored in the Metadata. Do not check when the
+    // Cluster was just updated with a new value for view_change_uuid and needs
+    // to be restarted
+    auto view_change_uuid_cluster =
+        m_cluster->get_cluster_server()->get_sysvar_string(
+            "group_replication_view_change_uuid", "");
+
+    if (view_change_uuid_cluster != m_cluster->get_view_change_uuid()) {
+      console->print_info(
+          "Updating group_replication_view_change_uuid in the Cluster's "
+          "metadata...");
+
+      m_cluster->get_metadata_storage()->update_cluster_attribute(
+          m_cluster->get_id(), "group_replication_view_change_uuid",
+          shcore::Value(view_change_uuid_cluster));
+    }
   }
 }
 
@@ -829,7 +884,7 @@ void Rescan::ensure_recovery_accounts_match() {
 
           // Generate the recovery account string for this instance
           std::string recovery_user_generated =
-              m_cluster->make_replication_user_name(
+              Cluster_impl::make_replication_user_name(
                   instance->get_server_id(),
                   old_prefix ? mysqlshdk::gr::k_group_recovery_old_user_prefix
                              : mysqlshdk::gr::k_group_recovery_user_prefix);
@@ -931,7 +986,9 @@ shcore::Value Rescan::execute() {
   m_cluster->ensure_metadata_has_server_id(*m_cluster->get_cluster_server());
 
   // Check if group_replication_view_change_uuid is set on the Cluster and all
-  // of its members when running MySQL >= 8.0.27
+  // of its members when running MySQL >= 8.0.27. Also verify the Metadata
+  // consistency regarding group_replication_view_change_uuid when already in
+  // use
   if (m_is_view_change_uuid_supported) {
     auto view_change_uuid = m_cluster->get_primary_master()->get_sysvar_string(
         "group_replication_view_change_uuid", "");
