@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -22,16 +22,20 @@
  */
 
 #include "modules/adminapi/cluster/status.h"
+
 #include <algorithm>
+
 #include "modules/adminapi/cluster_set/cluster_set_impl.h"
 #include "modules/adminapi/common/common.h"
 #include "modules/adminapi/common/common_status.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "modules/adminapi/common/parallel_applier_options.h"
+#include "modules/adminapi/common/server_features.h"
 #include "modules/adminapi/common/sql.h"
 #include "mysqlshdk/libs/mysql/clone.h"
 #include "mysqlshdk/libs/mysql/group_replication.h"
 #include "mysqlshdk/libs/mysql/repl_config.h"
+#include "mysqlshdk/libs/utils/debug.h"
 
 namespace mysqlsh {
 namespace dba {
@@ -956,6 +960,97 @@ void check_transaction_size_limit(shcore::Array_t issues, Instance *instance,
   }
 }
 
+void check_auth_type_instance_ssl(shcore::Array_t issues,
+                                  const Instance &instance,
+                                  const Cluster_impl &cluster) {
+  auto auth_type = cluster.query_cluster_auth_type();
+  if (auth_type == Replication_auth_type::PASSWORD) return;
+
+  // if we can be more specific
+  switch (auth_type) {
+    case Replication_auth_type::CERT_SUBJECT:
+    case Replication_auth_type::CERT_SUBJECT_PASSWORD:
+      if (cluster.query_cluster_instance_auth_cert_subject(instance).empty()) {
+        auto msg = shcore::str_format(
+            "WARNING: memberAuthType is set to '%s' but there's no "
+            "'certSubject' configured for this instance, which prevents all "
+            "other instances from reaching it, compromising the Cluster. "
+            "Please remove the instance from the Cluster and use the most "
+            "recent version of the shell to re-add it back.",
+            to_string(auth_type).c_str());
+        issues->push_back(shcore::Value(std::move(msg)));
+        return;
+      }
+    default:
+      break;
+  }
+
+  // check if instance user was created correctly
+  bool has_issuer{false}, has_subject{false};
+  {
+    auto instance_repl_account =
+        cluster.get_metadata_storage()->get_instance_repl_account(
+            instance.get_uuid(), Cluster_type::GROUP_REPLICATION);
+
+    if (std::get<0>(instance_repl_account).empty()) {
+      // if this happens, a warning is already shown to the user in
+      // validate_instance_recovery_user(), so there's no point showing a second
+      // error message related to the same thing, even though it almost surely
+      // means that the user was created with an older version of the shell
+      return;
+    }
+
+    if (auto server = cluster.get_cluster_server(); server) {
+      auto res = server->queryf(
+          "SELECT coalesce(length(x509_issuer)), "
+          "coalesce(length(x509_subject)) FROM mysql.user WHERE (user = ?)",
+          std::get<0>(instance_repl_account));
+
+      if (auto row = res->fetch_one()) {
+        has_issuer = row->get_int(0) > 0;
+        has_subject = row->get_int(1) > 0;
+      }
+    }
+  }
+
+  DBUG_EXECUTE_IF("fail_recovery_user_status_check_issuer",
+                  { has_issuer = false; });
+  DBUG_EXECUTE_IF("fail_recovery_user_status_check_subject",
+                  { has_subject = false; });
+
+  switch (auth_type) {
+    case Replication_auth_type::CERT_ISSUER:
+    case Replication_auth_type::CERT_ISSUER_PASSWORD:
+      if (!has_issuer) {
+        auto msg = shcore::str_format(
+            "WARNING: memberAuthType is set to '%s' but there's no "
+            "'certIssuer' configured for this instance recovery user, which "
+            "prevents all other instances from reaching it, compromising the "
+            "Cluster. Please remove the instance from the Cluster and use the "
+            "most recent version of the shell to re-add it back.",
+            to_string(auth_type).c_str());
+        issues->push_back(shcore::Value(std::move(msg)));
+      }
+      break;
+    case Replication_auth_type::CERT_SUBJECT:
+    case Replication_auth_type::CERT_SUBJECT_PASSWORD:
+      if (!has_issuer || !has_subject) {
+        auto msg = shcore::str_format(
+            "WARNING: memberAuthType is set to '%s' but there's no "
+            "'certIssuer' and/or 'certSubject' configured for this instance "
+            "recovery user, which prevents all other instances from reaching "
+            "it, compromising the Cluster. Please remove the instance from the "
+            "Cluster and use the most recent version of the shell to re-add it "
+            "back.",
+            to_string(auth_type).c_str());
+        issues->push_back(shcore::Value(std::move(msg)));
+      }
+      break;
+    default:
+      break;
+  }
+}
+
 shcore::Array_t instance_diagnostics(
     Instance *instance, const Cluster_impl *cluster,
     const Instance_metadata_info &instance_md,
@@ -1054,6 +1149,9 @@ shcore::Array_t instance_diagnostics(
                                  cluster_transaction_size_limit);
   }
 
+  // check if instance (if secondary) has a seemingly correct SSL settings
+  if (instance) check_auth_type_instance_ssl(issues, *instance, *cluster);
+
   return issues;
 }
 
@@ -1104,8 +1202,40 @@ void check_comm_protocol_upgrade_possible(
   }
 }
 
-shcore::Array_t group_diagnostics(
-    const Cluster_impl *cluster,
+void check_view_change_uuid_md(shcore::Array_t issues,
+                               const Cluster_impl &cluster) {
+  auto group_instance = cluster.get_cluster_server();
+
+  // If there's primary we can't check anything, exit
+  if (!group_instance) return;
+
+  // Check if group_replication_view_change_uuid is set on the cluster and if
+  // so, if it matches the value stored in the Metadata
+  auto view_change_uuid = cluster.get_cluster_server()->get_sysvar_string(
+      "group_replication_view_change_uuid", "");
+
+  // Not in use, exit
+  if (view_change_uuid.empty() || view_change_uuid == "AUTOMATIC") return;
+
+  auto md_view_change_uuid = cluster.get_view_change_uuid();
+
+  if (md_view_change_uuid.empty()) {
+    issues->push_back(shcore::Value(
+        "WARNING: The Cluster's group_replication_view_change_uuid is not "
+        "stored in the Metadata. Please use <Cluster>.rescan() to update the "
+        "metadata."));
+  } else {
+    if (md_view_change_uuid != view_change_uuid) {
+      issues->push_back(shcore::Value(
+          "WARNING: The Cluster's group_replication_view_change_uuid value "
+          "in use does not match the value stored in the Metadata. Please "
+          "use <Cluster>.rescan() to update the metadata."));
+    }
+  }
+}
+
+shcore::Array_t cluster_diagnostics(
+    const Cluster_impl &cluster,
     const std::vector<mysqlshdk::gr::Member> &member_info,
     const mysqlshdk::utils::Version &protocol_version) {
   shcore::Array_t issues = shcore::make_array();
@@ -1113,7 +1243,10 @@ shcore::Array_t group_diagnostics(
   if (protocol_version && !member_info.empty())
     check_comm_protocol_upgrade_possible(issues, member_info, protocol_version);
 
-  switch (cluster->cluster_availability()) {
+  // Verify Metadata consistency regarding group_replication_view_change_uuid
+  check_view_change_uuid_md(issues, cluster);
+
+  switch (cluster.cluster_availability()) {
     case Cluster_availability::NO_QUORUM:
       issues->push_back(
           shcore::Value("ERROR: Could not find ONLINE members forming a "
@@ -1135,7 +1268,7 @@ shcore::Array_t group_diagnostics(
       break;
   }
 
-  if (cluster->is_fenced_from_writes()) {
+  if (cluster.is_fenced_from_writes()) {
     issues->push_back(
         shcore::Value("WARNING: Cluster is fenced from Write traffic. Use "
                       "cluster.unfenceWrites() to unfence the Cluster."));
@@ -1454,9 +1587,19 @@ shcore::Dictionary_t Status::get_topology(
           super_read_only, minfo, self_state, parallel_applier_options,
           *m_cluster_transaction_size_limit);
 
-      if (offline_mode.value_or(false))
+      if (offline_mode.value_or(false)) {
         issues->push_back(
-            shcore::Value("WARNING: Instance has offline_mode enabled."));
+            shcore::Value("WARNING: Instance has 'offline_mode' enabled."));
+      } else if (instance && instance->is_set_persist_supported()) {
+        auto value = instance->get_persisted_value("offline_mode");
+        if (value.has_value() && shcore::str_caseeq(*value, "ON")) {
+          issues->push_back(shcore::Value(
+              "WARNING: Instance has 'offline_mode' enabled and persisted. In "
+              "the event that this instance becomes a primary, Shell or other "
+              "members will be prevented from connecting to it disrupting the "
+              "Cluster's normal functioning."));
+        }
+      }
 
       if (instance) {
         auto ret_val = validate_instance_recovery_user(
@@ -1577,23 +1720,38 @@ shcore::Dictionary_t Status::collect_replicaset_status() {
     //   different communication stack.
     if (group_instance) {
       (*ret)[kCommunicationStack] =
-          shcore::Value(m_cluster.get_communication_stack());
+          shcore::Value(get_communication_stack(*group_instance));
+    }
+
+    // Add the value of paxosSingleLeader, when supported
+    if (group_instance &&
+        supports_paxos_single_leader(group_instance->get_version())) {
+      auto value = get_paxos_single_leader_enabled(*group_instance);
+
+      if (value.has_value()) {
+        std::string_view paxos_single_leader = *value == true ? "ON" : "OFF";
+
+        (*ret)[kPaxosSingleLeader] = shcore::Value(paxos_single_leader);
+      }
     }
   }
 
-  auto ssl_mode = group_instance ? group_instance->get_sysvar_string(
-                                       "group_replication_ssl_mode", "")
-                                 : "";
-  if (!ssl_mode.empty()) {
-    (*ret)["ssl"] = shcore::Value(ssl_mode);
+  {
+    auto ssl_mode = group_instance ? group_instance->get_sysvar_string(
+                                         "group_replication_ssl_mode", "")
+                                   : "";
+    if (!ssl_mode.empty()) {
+      (*ret)["ssl"] = shcore::Value(ssl_mode);
+    }
   }
 
-  bool single_primary;
-  std::string view_id;
-  bool has_quorum;
+  bool single_primary = true;
   std::vector<mysqlshdk::gr::Member> member_info;
 
   if (group_instance && gr_running) {
+    bool has_quorum = true;
+    std::string view_id;
+
     member_info = mysqlshdk::gr::get_members(*group_instance, &single_primary,
                                              &has_quorum, &view_id);
 
@@ -1634,41 +1792,54 @@ shcore::Dictionary_t Status::collect_replicaset_status() {
     }
   }
 
-  {
-    auto issues = group_diagnostics(&m_cluster, member_info, protocol_version);
+  auto issues = cluster_diagnostics(m_cluster, member_info, protocol_version);
 
-    // Get the Cluster's transaction size_limit stored in the Metadata if the
-    // Cluster is standalone or a Primary. Otherwise, it's a Replica so it
-    // should be the value of the primary member
-    shcore::Value value;
-    bool is_replica_cluster =
-        m_cluster.is_cluster_set_member() && !m_cluster.is_primary_cluster();
+  // Get the Cluster's transaction size_limit stored in the Metadata if the
+  // Cluster is standalone or a Primary. Otherwise, it's a Replica so it
+  // should be the value of the primary member
+  shcore::Value value;
+  bool is_replica_cluster =
+      m_cluster.is_cluster_set_member() && !m_cluster.is_primary_cluster();
 
-    if (group_instance) {
-      m_cluster_transaction_size_limit =
-          m_cluster.get_cluster_server()
-              ->get_sysvar_int(kGrTransactionSizeLimit)
-              .value_or(0);
-    }
-
-    if (!is_replica_cluster) {
-      if (!m_cluster.get_metadata_storage()->query_cluster_attribute(
-              m_cluster.get_id(), k_cluster_attribute_transaction_size_limit,
-              &value)) {
-        issues->push_back(
-            shcore::Value("WARNING: Cluster's transaction size limit is not "
-                          "registered in the metadata. Use "
-                          "cluster.rescan() to update the metadata."));
-
-      } else {
-        m_cluster_transaction_size_limit = value.as_int();
-      }
-    }
-
-    if (issues && !issues->empty())
-      (*ret)["clusterErrors"] = shcore::Value(issues);
+  if (group_instance) {
+    m_cluster_transaction_size_limit =
+        m_cluster.get_cluster_server()
+            ->get_sysvar_int(kGrTransactionSizeLimit)
+            .value_or(0);
   }
 
+  if (!is_replica_cluster) {
+    if (!m_cluster.get_metadata_storage()->query_cluster_attribute(
+            m_cluster.get_id(), k_cluster_attribute_transaction_size_limit,
+            &value)) {
+      issues->push_back(
+          shcore::Value("WARNING: Cluster's transaction size limit is not "
+                        "registered in the metadata. Use "
+                        "cluster.rescan() to update the metadata."));
+
+    } else {
+      m_cluster_transaction_size_limit = value.as_int();
+    }
+  }
+
+  // If the cluster is operating in multi-primary mode and paxosSingleLeader
+  // is enabled, add a NOTE informing the Cluster won't use a single
+  // consensus leader in that case
+  bool is_active_multi_primary =
+      group_instance && gr_running && !single_primary;
+
+  if (is_active_multi_primary &&
+      supports_paxos_single_leader(group_instance->get_version()) &&
+      get_paxos_single_leader_enabled(*group_instance).value_or(false)) {
+    issues->push_back(
+        shcore::Value("NOTE: The Cluster is configured to use a Single "
+                      "Consensus Leader, however, this setting is ineffective "
+                      "since the Cluster is operating in multi-primary mode."));
+  }
+
+  if (issues && !issues->empty())
+    (*ret)["clusterErrors"] = shcore::Value(issues);
+
   (*ret)["topology"] = shcore::Value(get_topology(member_info));
 
   return ret;
@@ -1746,7 +1917,7 @@ shcore::Value Status::get_default_replicaset_status() {
       warning.append(ManagedInstance::describe(
           static_cast<ManagedInstance::State>(state.source_state)));
       warning.append(" state");
-      (*replicaset_dict)["warning"] = shcore::Value(warning);
+      (*replicaset_dict)["warning"] = shcore::Value(std::move(warning));
     }
   }
 
