@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -34,6 +34,7 @@
 #include "modules/adminapi/common/cluster_types.h"
 #include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/metadata_storage.h"
+#include "modules/adminapi/common/server_features.h"
 #include "modules/adminapi/common/sql.h"
 #include "modules/adminapi/common/validations.h"
 #include "modules/adminapi/mod_dba.h"
@@ -111,15 +112,54 @@ std::string to_string(Cluster_ssl_mode ssl_mode) {
   return {};
 }
 
-Cluster_ssl_mode to_cluster_ssl_mode(const std::string &mode) {
-  if (shcore::str_caseeq("AUTO", mode)) return Cluster_ssl_mode::AUTO;
-  if (shcore::str_caseeq("DISABLED", mode)) return Cluster_ssl_mode::DISABLED;
-  if (shcore::str_caseeq("REQUIRED", mode)) return Cluster_ssl_mode::REQUIRED;
-  if (shcore::str_caseeq("VERIFY_CA", mode)) return Cluster_ssl_mode::VERIFY_CA;
-  if (shcore::str_caseeq("VERIFY_IDENTITY", mode))
+Cluster_ssl_mode to_cluster_ssl_mode(std::string_view mode) {
+  if (shcore::str_caseeq(kClusterSSLModeAuto, mode))
+    return Cluster_ssl_mode::AUTO;
+  if (shcore::str_caseeq(kClusterSSLModeDisabled, mode))
+    return Cluster_ssl_mode::DISABLED;
+  if (shcore::str_caseeq(kClusterSSLModeRequired, mode))
+    return Cluster_ssl_mode::REQUIRED;
+  if (shcore::str_caseeq(kClusterSSLModeVerifyCA, mode))
+    return Cluster_ssl_mode::VERIFY_CA;
+  if (shcore::str_caseeq(kClusterSSLModeVerifyIdentity, mode))
     return Cluster_ssl_mode::VERIFY_IDENTITY;
 
-  throw std::runtime_error("Unsupported Cluster SSL-mode: " + mode);
+  throw std::runtime_error(
+      shcore::str_format("Unsupported Cluster SSL-mode: %.*s",
+                         static_cast<int>(mode.size()), mode.data()));
+}
+
+std::string to_string(Replication_auth_type auth) {
+  switch (auth) {
+    case Replication_auth_type::PASSWORD:
+      return kReplicationMemberAuthPassword;
+    case Replication_auth_type::CERT_ISSUER:
+      return kReplicationMemberAuthCertIssuer;
+    case Replication_auth_type::CERT_SUBJECT:
+      return kReplicationMemberAuthCertSubject;
+    case Replication_auth_type::CERT_ISSUER_PASSWORD:
+      return kReplicationMemberAuthCertIssuerPassword;
+    case Replication_auth_type::CERT_SUBJECT_PASSWORD:
+      return kReplicationMemberAuthCertSubjectPassword;
+  }
+  return {};
+}
+
+Replication_auth_type to_replication_auth_type(std::string_view auth) {
+  if (shcore::str_caseeq(kReplicationMemberAuthPassword, auth))
+    return Replication_auth_type::PASSWORD;
+  if (shcore::str_caseeq(kReplicationMemberAuthCertIssuer, auth))
+    return Replication_auth_type::CERT_ISSUER;
+  if (shcore::str_caseeq(kReplicationMemberAuthCertSubject, auth))
+    return Replication_auth_type::CERT_SUBJECT;
+  if (shcore::str_caseeq(kReplicationMemberAuthCertIssuerPassword, auth))
+    return Replication_auth_type::CERT_ISSUER_PASSWORD;
+  if (shcore::str_caseeq(kReplicationMemberAuthCertSubjectPassword, auth))
+    return Replication_auth_type::CERT_SUBJECT_PASSWORD;
+
+  throw std::runtime_error(
+      shcore::str_format("Unsupported replication member authentication: %.*s",
+                         static_cast<int>(auth.size()), auth.data()));
 }
 
 std::string get_mysqlprovision_error_string(
@@ -426,6 +466,140 @@ void validate_instance_ssl_mode(Cluster_type type,
   }
 }
 
+void validate_instance_member_auth_type(
+    const mysqlshdk::mysql::IInstance &instance, bool is_replica_cluster,
+    Cluster_ssl_mode ssl_mode, std::string_view ssl_mode_option,
+    Replication_auth_type auth_type) {
+  if (auth_type == Replication_auth_type::PASSWORD) return;
+
+  if (ssl_mode == Cluster_ssl_mode::DISABLED) {
+    if (is_replica_cluster)
+      throw shcore::Exception::runtime_error(shcore::str_format(
+          "Instance '%s' has the \"%.*s\" option with the value "
+          "'DISABLED', which isn't compatible with the \"memberAuthType\" "
+          "value of '%s' inherited from the ClusterSet.",
+          instance.descr().c_str(), static_cast<int>(ssl_mode_option.length()),
+          ssl_mode_option.data(), to_string(auth_type).c_str()));
+
+    throw shcore::Exception::runtime_error(shcore::str_format(
+        "Instance '%s' has the \"%.*s\" option with the value "
+        "'DISABLED', which means that \"memberAuthType\" only accepts the "
+        "value 'PASSWORD'.",
+        instance.descr().c_str(), static_cast<int>(ssl_mode_option.length()),
+        ssl_mode_option.data()));
+  }
+
+  if (!instance.is_ssl_enabled()) {
+    std::string_view cluster_set_ctx{
+        is_replica_cluster ? " (inherited from the ClusterSet)" : ""};
+
+    throw shcore::Exception::runtime_error(shcore::str_format(
+        "Instance '%s' does not support TLS, which means that "
+        "\"memberAuthType\"%.*s with value '%s' isn't supported.",
+        instance.descr().c_str(), static_cast<int>(cluster_set_ctx.length()),
+        cluster_set_ctx.data(), to_string(auth_type).c_str()));
+  }
+
+  // verify the certificates
+  ensure_certificates_set(instance, auth_type);
+}
+
+void validate_instance_member_auth_options(std::string_view context,
+                                           bool is_replica_cluster,
+                                           Replication_auth_type auth_type,
+                                           std::string_view cert_subject) {
+  std::string_view cluster_set_ctx{
+      is_replica_cluster ? " (inherited from the ClusterSet)" : ""};
+
+  switch (auth_type) {
+    case Replication_auth_type::PASSWORD:
+    case Replication_auth_type::CERT_ISSUER:
+    case Replication_auth_type::CERT_ISSUER_PASSWORD:
+      if (!cert_subject.empty())
+        mysqlsh::current_console()->print_note(shcore::str_format(
+            "The %.*s's SSL mode%.*s is set to %s, which makes the "
+            "'certSubject' option not required. The option will be ignored.",
+            static_cast<int>(context.length()), context.data(),
+            static_cast<int>(cluster_set_ctx.length()), cluster_set_ctx.data(),
+            to_string(auth_type).c_str()));
+      break;
+
+    case Replication_auth_type::CERT_SUBJECT:
+    case Replication_auth_type::CERT_SUBJECT_PASSWORD:
+      if (cert_subject.empty())
+        throw shcore::Exception(
+            shcore::str_format(
+                "The %.*s's SSL mode%.*s is set to %s but the 'certSubject' "
+                "option for the instance wasn't supplied.",
+                static_cast<int>(context.length()), context.data(),
+                static_cast<int>(cluster_set_ctx.length()),
+                cluster_set_ctx.data(), to_string(auth_type).c_str()),
+            SHERR_DBA_MISSING_CERT_OPTION);
+      break;
+    default:
+      break;
+  }
+}
+
+void validate_instance_member_auth_options(std::string_view context,
+                                           Replication_auth_type auth_type,
+                                           std::string_view cert_issuer,
+                                           std::string_view cert_subject) {
+  if (auth_type == Replication_auth_type::PASSWORD) {
+    if (cert_issuer.empty() && cert_subject.empty()) return;
+
+    mysqlsh::current_console()->print_note(shcore::str_format(
+        "The %.*s's SSL mode is set to %s, which makes both the 'certIssuer' "
+        "and 'certSubject' options not required. Both options will be ignored.",
+        static_cast<int>(context.length()), context.data(),
+        to_string(auth_type).c_str()));
+    return;
+  }
+
+  switch (auth_type) {
+    case Replication_auth_type::CERT_ISSUER:
+    case Replication_auth_type::CERT_ISSUER_PASSWORD:
+    case Replication_auth_type::CERT_SUBJECT:
+    case Replication_auth_type::CERT_SUBJECT_PASSWORD:
+      if (cert_issuer.empty())
+        throw shcore::Exception(
+            shcore::str_format("The %.*s's SSL mode is set to %s but the "
+                               "'certIssuer' option wasn't supplied.",
+                               static_cast<int>(context.length()),
+                               context.data(), to_string(auth_type).c_str()),
+            SHERR_DBA_MISSING_CERT_OPTION);
+      break;
+    default:
+      break;
+  }
+
+  switch (auth_type) {
+    case Replication_auth_type::CERT_ISSUER:
+    case Replication_auth_type::CERT_ISSUER_PASSWORD:
+      if (!cert_subject.empty())
+        mysqlsh::current_console()->print_note(shcore::str_format(
+            "The %.*s's SSL mode is set to %s, which makes the 'certSubject' "
+            "option not required. The option will be ignored.",
+            static_cast<int>(context.length()), context.data(),
+            to_string(auth_type).c_str()));
+      break;
+
+    case Replication_auth_type::CERT_SUBJECT:
+    case Replication_auth_type::CERT_SUBJECT_PASSWORD:
+      if (cert_subject.empty())
+        throw shcore::Exception(
+            shcore::str_format(
+                "The %.*s's SSL mode is set to %s but the 'certSubject' option "
+                "for the instance wasn't supplied.",
+                static_cast<int>(context.length()), context.data(),
+                to_string(auth_type).c_str()),
+            SHERR_DBA_MISSING_CERT_OPTION);
+      break;
+    default:
+      break;
+  }
+}
+
 /**
  * Gets the list of instances belonging to the GR group which are not
  * registered on the metadata for a specific ReplicaSet.
@@ -1126,15 +1300,16 @@ bool prompt_super_read_only(const mysqlshdk::mysql::IInstance &instance,
 void dump_table(const std::vector<std::string> &column_names,
                 const std::vector<std::string> &column_labels,
                 shcore::Value::Array_type_ref documents) {
-  std::vector<uint64_t> max_lengths;
-
   size_t field_count = column_names.size();
 
+  std::vector<size_t> max_lengths;
+  max_lengths.reserve(field_count);
+
   // Updates the max_length array with the maximum length between column name,
   // min column length and column max length
   for (size_t field_index = 0; field_index < field_count; field_index++) {
     max_lengths.push_back(0);
-    max_lengths[field_index] = std::max<uint64_t>(
+    max_lengths[field_index] = std::max<size_t>(
         max_lengths[field_index], column_labels[field_index].size());
   }
 
@@ -1143,7 +1318,7 @@ void dump_table(const std::vector<std::string> &column_names,
     for (auto map : *documents) {
       auto document = map.as_map();
       for (size_t field_index = 0; field_index < field_count; field_index++)
-        max_lengths[field_index] = std::max<uint64_t>(
+        max_lengths[field_index] = std::max<size_t>(
             max_lengths[field_index],
             document->get_string(column_names[field_index]).size());
     }
@@ -1151,12 +1326,11 @@ void dump_table(const std::vector<std::string> &column_names,
 
   //-----------
 
-  size_t index = 0;
   std::vector<std::string> formats(field_count, "%-");
 
   // Calculates the max column widths and constructs the separator line.
   std::string separator("+");
-  for (index = 0; index < field_count; index++) {
+  for (size_t index = 0; index < field_count; index++) {
     // Creates the format string to print each field
     formats[index].append(std::to_string(max_lengths[index]));
     if (index == field_count - 1)
@@ -1164,9 +1338,8 @@ void dump_table(const std::vector<std::string> &column_names,
     else
       formats[index].append("s | ");
 
-    std::string field_separator(max_lengths[index] + 2, '-');
-    field_separator.append("+");
-    separator.append(field_separator);
+    separator.append(max_lengths[index] + 2, '-');
+    separator.append(1, '+');
   }
   separator.append("\n");
 
@@ -1175,7 +1348,7 @@ void dump_table(const std::vector<std::string> &column_names,
   console->print(separator);
   console->print("| ");
 
-  for (index = 0; index < field_count; index++) {
+  for (size_t index = 0; index < field_count; index++) {
     std::string data = shcore::str_format(formats[index].c_str(),
                                           column_labels[index].c_str());
     console->print(data.c_str());
@@ -1232,6 +1405,8 @@ ConfigureInstanceAction get_configure_instance_action(
     ret_val = ConfigureInstanceAction::UPDATE_SERVER_DYNAMIC;
   else if (action == "server_update+restart")
     ret_val = ConfigureInstanceAction::UPDATE_SERVER_STATIC;
+  else if (action == "remove_opt+restart")
+    ret_val = ConfigureInstanceAction::REMOVE_OPTION_RESTART;
   else if (action == "restart")
     ret_val = ConfigureInstanceAction::RESTART;
   else
@@ -1250,65 +1425,66 @@ ConfigureInstanceAction get_configure_instance_action(
  */
 void print_validation_results(const shcore::Value::Map_type_ref &result,
                               bool print_note) {
+  if (!result->has_key("config_errors")) return;
+
   auto console = mysqlsh::current_console();
+  console->print_note("Some configuration options need to be fixed:");
 
-  if (result->has_key("config_errors")) {
-    console->print_note("Some configuration options need to be fixed:");
-
-    auto config_errors = result->get_array("config_errors");
-
-    if (print_note) {
-      for (auto option : *config_errors) {
-        auto opt_map = option.as_map();
-        ConfigureInstanceAction action =
-            get_configure_instance_action(*opt_map);
-        std::string note;
-        switch (action) {
-          case ConfigureInstanceAction::UPDATE_SERVER_AND_CONFIG_DYNAMIC: {
-            note = "Update the server variable and the config file";
-            break;
-          }
-          case ConfigureInstanceAction::UPDATE_SERVER_AND_CONFIG_STATIC: {
-            note = "Update the config file and restart the server";
-            break;
-          }
-          case ConfigureInstanceAction::UPDATE_CONFIG: {
-            note = "Update the config file";
-            break;
-          }
-          case ConfigureInstanceAction::UPDATE_SERVER_DYNAMIC: {
-            note = "Update the server variable";
-            break;
-          }
-          case ConfigureInstanceAction::UPDATE_SERVER_STATIC: {
-            note = "Update read-only variable and restart the server";
-            break;
-          }
-          case ConfigureInstanceAction::RESTART: {
-            note = "Restart the server";
-            break;
-          }
-          default: {
-            note = "Undefined action";
-            break;
-          }
-        }
-        (*opt_map)["note"] = shcore::Value(note);
-      }
-
-      dump_table({"option", "current", "required", "note"},
-                 {"Variable", "Current Value", "Required Value", "Note"},
-                 config_errors);
-    } else {
-      dump_table({"option", "current", "required"},
-                 {"Variable", "Current Value", "Required Value"},
-                 config_errors);
-    }
+  auto config_errors = result->get_array("config_errors");
 
+  if (print_note) {
     for (auto option : *config_errors) {
       auto opt_map = option.as_map();
-      opt_map->erase("note");
+      ConfigureInstanceAction action = get_configure_instance_action(*opt_map);
+      std::string note;
+      switch (action) {
+        case ConfigureInstanceAction::UPDATE_SERVER_AND_CONFIG_DYNAMIC: {
+          note = "Update the server variable and the config file";
+          break;
+        }
+        case ConfigureInstanceAction::UPDATE_SERVER_AND_CONFIG_STATIC: {
+          note = "Update the config file and restart the server";
+          break;
+        }
+        case ConfigureInstanceAction::UPDATE_CONFIG: {
+          note = "Update the config file";
+          break;
+        }
+        case ConfigureInstanceAction::UPDATE_SERVER_DYNAMIC: {
+          note = "Update the server variable";
+          break;
+        }
+        case ConfigureInstanceAction::UPDATE_SERVER_STATIC: {
+          note = "Update read-only variable and restart the server";
+          break;
+        }
+        case ConfigureInstanceAction::REMOVE_OPTION_RESTART: {
+          note = "Remove the option and restart the server";
+          break;
+        }
+        case ConfigureInstanceAction::RESTART: {
+          note = "Restart the server";
+          break;
+        }
+        default: {
+          note = "Undefined action";
+          break;
+        }
+      }
+      (*opt_map)["note"] = shcore::Value(std::move(note));
     }
+
+    dump_table({"option", "current", "required", "note"},
+               {"Variable", "Current Value", "Required Value", "Note"},
+               config_errors);
+  } else {
+    dump_table({"option", "current", "required"},
+               {"Variable", "Current Value", "Required Value"}, config_errors);
+  }
+
+  for (auto option : *config_errors) {
+    auto opt_map = option.as_map();
+    opt_map->erase("note");
   }
 }
 
@@ -2139,6 +2315,39 @@ Cluster_check_info get_cluster_check_info(const MetadataStorage &metadata,
   return state;
 }
 
+mysqlshdk::db::Ssl_options prepare_replica_ssl_options(
+    const mysqlshdk::mysql::IInstance &instance, Cluster_ssl_mode ssl_mode,
+    Replication_auth_type auth_type) {
+  mysqlshdk::db::Ssl_options ssl_options;
+
+  if (ssl_mode == Cluster_ssl_mode::VERIFY_CA ||
+      ssl_mode == Cluster_ssl_mode::VERIFY_IDENTITY ||
+      auth_type != Replication_auth_type::PASSWORD) {
+    ssl_options = read_ssl_client_options(
+        instance, ssl_mode != Cluster_ssl_mode::REQUIRED ||
+                      auth_type != Replication_auth_type::PASSWORD);
+  }
+  switch (ssl_mode) {
+    case Cluster_ssl_mode::NONE:
+    case Cluster_ssl_mode::AUTO:
+      assert(0);  // these are supposed to be resolved before reaching here
+      [[fallthrough]];
+    case Cluster_ssl_mode::DISABLED:
+      ssl_options.set_mode(mysqlshdk::db::Ssl_mode::Disabled);
+      break;
+    case Cluster_ssl_mode::REQUIRED:
+      ssl_options.set_mode(mysqlshdk::db::Ssl_mode::Required);
+      break;
+    case Cluster_ssl_mode::VERIFY_CA:
+      ssl_options.set_mode(mysqlshdk::db::Ssl_mode::VerifyCa);
+      break;
+    case Cluster_ssl_mode::VERIFY_IDENTITY:
+      ssl_options.set_mode(mysqlshdk::db::Ssl_mode::VerifyIdentity);
+      break;
+  }
+  return ssl_options;
+}
+
 namespace cluster_topology_executor_ops {
 void validate_add_rejoin_options(Group_replication_options options,
                                  const std::string &communication_stack) {
@@ -2184,8 +2393,7 @@ void check_comm_stack_support(
     Group_replication_options *gr_options,
     const std::string &communication_stack) {
   if (communication_stack == kCommunicationStackMySQL &&
-      target_instance->get_version() <
-          k_mysql_communication_stack_initial_version) {
+      !supports_mysql_communication_stack(target_instance->get_version())) {
     auto console = mysqlsh::current_console();
     console->print_error("Cannot join instance '" + target_instance->descr() +
                          "' to Cluster: The Group Replication protocol stack "
@@ -2200,8 +2408,7 @@ void check_comm_stack_support(
 
   // Set the communication stack protocol to be used only if the target
   // instance supports it
-  if (target_instance->get_version() >=
-      k_mysql_communication_stack_initial_version) {
+  if (supports_mysql_communication_stack(target_instance->get_version())) {
     gr_options->communication_stack = communication_stack;
   }
 }
