@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -462,7 +462,8 @@ std::shared_ptr<mysqlshdk::db::IResult> MetadataStorage::execute_sql(
   try {
     ret_val = m_md_server->query(sql);
   } catch (const shcore::Error &err) {
-    log_warning("While querying metadata: %s", err.format().c_str());
+    log_warning("While querying metadata: %s\n\t%s", err.format().c_str(),
+                sql.c_str());
     if (CR_SERVER_GONE_ERROR == err.code()) {
       log_debug("The Metadata server is inaccessible");
       throw shcore::Exception::metadata_error("The Metadata is inaccessible");
@@ -552,13 +553,15 @@ bool MetadataStorage::get_cluster(const Cluster_id &cluster_id,
 }
 
 bool MetadataStorage::query_cluster_attribute(const Cluster_id &cluster_id,
-                                              const std::string &attribute,
+                                              std::string_view attribute,
                                               shcore::Value *out_value) const {
   auto result = execute_sql(
-      shcore::sqlstring("SELECT attributes->'$." + attribute +
-                            "' FROM mysql_innodb_cluster_metadata.clusters"
-                            " WHERE cluster_id=?",
-                        0)
+      shcore::sqlstring(
+          shcore::str_format(
+              "SELECT attributes->'$.%.*s' FROM "
+              "mysql_innodb_cluster_metadata.clusters WHERE cluster_id=?",
+              static_cast<int>(attribute.length()), attribute.data()),
+          0)
       << cluster_id);
 
   if (auto row = result->fetch_one()) {
@@ -571,15 +574,15 @@ bool MetadataStorage::query_cluster_attribute(const Cluster_id &cluster_id,
 }
 
 void MetadataStorage::update_cluster_attribute(const Cluster_id &cluster_id,
-                                               const std::string &attribute,
+                                               std::string_view attribute,
                                                const shcore::Value &value) {
   if (value) {
     shcore::sqlstring query(
-        "UPDATE mysql_innodb_cluster_metadata.clusters"
-        " SET attributes = json_set(attributes, '$." +
-            attribute +
-            "', CAST(? as JSON))"
-            " WHERE cluster_id = ?",
+        shcore::str_format("UPDATE mysql_innodb_cluster_metadata.clusters SET "
+                           "attributes = json_set(attributes, '$.%.*s', CAST(? "
+                           "as JSON)) WHERE cluster_id = ?",
+                           static_cast<int>(attribute.length()),
+                           attribute.data()),
         0);
 
     query << value.repr() << cluster_id;
@@ -587,11 +590,10 @@ void MetadataStorage::update_cluster_attribute(const Cluster_id &cluster_id,
     execute_sql(query);
   } else {
     shcore::sqlstring query(
-        "UPDATE mysql_innodb_cluster_metadata.clusters"
-        " SET attributes = json_remove(attributes, '$." +
-            attribute +
-            "')"
-            " WHERE cluster_id = ?",
+        shcore::str_format(
+            "UPDATE mysql_innodb_cluster_metadata.clusters SET attributes = "
+            "json_remove(attributes, '$.%.*s') WHERE cluster_id = ?",
+            static_cast<int>(attribute.length()), attribute.data()),
         0);
     query << cluster_id;
     query.done();
@@ -599,13 +601,14 @@ void MetadataStorage::update_cluster_attribute(const Cluster_id &cluster_id,
   }
 }
 
-void MetadataStorage::update_clusters_attribute(const std::string &attribute,
+void MetadataStorage::update_clusters_attribute(std::string_view attribute,
                                                 const shcore::Value &value) {
   if (value) {
     shcore::sqlstring query(
-        "UPDATE mysql_innodb_cluster_metadata.clusters"
-        " SET attributes = json_set(attributes, '$." +
-            attribute + "', CAST(? as JSON))",
+        shcore::str_format(
+            "UPDATE mysql_innodb_cluster_metadata.clusters SET attributes = "
+            "json_set(attributes, '$.%.*s', CAST(? as JSON))",
+            static_cast<int>(attribute.length()), attribute.data()),
         0);
 
     query << value.repr();
@@ -613,9 +616,10 @@ void MetadataStorage::update_clusters_attribute(const std::string &attribute,
     execute_sql(query);
   } else {
     shcore::sqlstring query(
-        "UPDATE mysql_innodb_cluster_metadata.clusters"
-        " SET attributes = json_remove(attributes, '$." +
-            attribute + "')",
+        shcore::str_format("UPDATE mysql_innodb_cluster_metadata.clusters SET "
+                           "attributes = json_remove(attributes, '$.%.*s')",
+                           static_cast<int>(attribute.length()),
+                           attribute.data()),
         0);
     query.done();
     execute_sql(query);
@@ -666,28 +670,26 @@ void MetadataStorage::update_cluster_capability(
 }
 
 void MetadataStorage::update_cluster_set_attribute(
-    const Cluster_set_id &clusterset_id, const std::string &attribute,
+    const Cluster_set_id &clusterset_id, std::string_view attribute,
     const shcore::Value &value) {
   shcore::sqlstring query;
 
   if (value) {
-    query = shcore::sqlstring(
-        "UPDATE mysql_innodb_cluster_metadata.clustersets"
-        " SET attributes = json_set(attributes, '$." +
-            attribute +
-            "', CAST(? as JSON))"
-            " WHERE clusterset_id = ?",
-        0);
+    auto stmt = shcore::str_format(
+        "UPDATE mysql_innodb_cluster_metadata.clustersets SET attributes = "
+        "json_set(attributes, '$.%.*s', CAST(? as JSON)) WHERE clusterset_id = "
+        "?",
+        static_cast<int>(attribute.length()), attribute.data());
 
+    query = shcore::sqlstring(stmt, 0);
     query << value.repr() << clusterset_id;
   } else {
-    query = shcore::sqlstring(
-        "UPDATE mysql_innodb_cluster_metadata.clustersets"
-        " SET attributes = json_remove(attributes, '$." +
-            attribute +
-            "')"
-            " WHERE clusterset_id = ?",
-        0);
+    auto stmt = shcore::str_format(
+        "UPDATE mysql_innodb_cluster_metadata.clustersets SET attributes = "
+        "json_remove(attributes, '$.%.*s') WHERE clusterset_id = ?",
+        static_cast<int>(attribute.length()), attribute.data());
+
+    query = shcore::sqlstring(stmt, 0);
     query << clusterset_id;
   }
 
@@ -696,14 +698,14 @@ void MetadataStorage::update_cluster_set_attribute(
 }
 
 bool MetadataStorage::query_cluster_set_attribute(
-    const Cluster_set_id &clusterset_id, const std::string &attribute,
+    const Cluster_set_id &clusterset_id, std::string_view attribute,
     shcore::Value *out_value) const {
-  auto result = execute_sql(
-      shcore::sqlstring("SELECT attributes->'$." + attribute +
-                            "' FROM mysql_innodb_cluster_metadata.clustersets"
-                            " WHERE clusterset_id=?",
-                        0)
-      << clusterset_id);
+  auto stmt = shcore::str_format(
+      "SELECT attributes->'$.%.*s' FROM "
+      "mysql_innodb_cluster_metadata.clustersets WHERE clusterset_id=?",
+      static_cast<int>(attribute.length()), attribute.data());
+
+  auto result = execute_sql(shcore::sqlstring(stmt, 0) << clusterset_id);
 
   if (auto row = result->fetch_one()) {
     if (!row->is_null(0)) {
@@ -847,15 +849,15 @@ Cluster_id MetadataStorage::create_cluster_record(Cluster_impl *cluster,
 
 Instance_id MetadataStorage::insert_instance(
     const Instance_metadata &instance) {
-  std::string addresses;
-  addresses = shcore::sqlstring("'mysqlClassic', ?", 0) << instance.endpoint;
+  auto addresses = ("'mysqlClassic', ?"_sql << instance.endpoint).str();
   if (!instance.xendpoint.empty())
-    addresses += shcore::sqlstring(", 'mysqlX', ?", 0) << instance.xendpoint;
+    addresses += (", 'mysqlX', ?"_sql << instance.xendpoint).str();
   if (!instance.grendpoint.empty())
-    addresses += shcore::sqlstring(", 'grLocal', ?", 0) << instance.grendpoint;
+    addresses += (", 'grLocal', ?"_sql << instance.grendpoint).str();
 
-  std::string attributes;
-  attributes = shcore::sqlstring("'server_id', ?", 0) << instance.server_id;
+  auto attributes = ("'server_id', ?"_sql << instance.server_id).str();
+  if (!instance.cert_subject.empty())
+    attributes += ("'cert_subject', ?"_sql << instance.cert_subject).str();
 
   shcore::sqlstring query;
   query = shcore::sqlstring(
@@ -878,15 +880,15 @@ Instance_id MetadataStorage::insert_instance(
 }
 
 void MetadataStorage::update_instance(const Instance_metadata &instance) {
-  std::string addresses;
-  addresses = shcore::sqlstring("'mysqlClassic', ?", 0) << instance.endpoint;
+  auto addresses = ("'mysqlClassic', ?"_sql << instance.endpoint).str();
   if (!instance.xendpoint.empty())
-    addresses += shcore::sqlstring(", 'mysqlX', ?", 0) << instance.xendpoint;
+    addresses += (", 'mysqlX', ?"_sql << instance.xendpoint).str();
   if (!instance.grendpoint.empty())
-    addresses += shcore::sqlstring(", 'grLocal', ?", 0) << instance.grendpoint;
+    addresses += (", 'grLocal', ?"_sql << instance.grendpoint).str();
 
-  std::string attributes;
-  attributes = shcore::sqlstring("'server_id', ?", 0) << instance.server_id;
+  auto attributes = ("'server_id', ?"_sql << instance.server_id).str();
+  if (!instance.cert_subject.empty())
+    attributes += ("'cert_subject', ?"_sql << instance.cert_subject).str();
 
   bool lookup_by_address = instance.id == 0 || instance.cluster_id.empty();
 
@@ -919,15 +921,15 @@ void MetadataStorage::update_instance(const Instance_metadata &instance) {
   execute_sql(query);
 }
 
-bool MetadataStorage::query_instance_attribute(const std::string &uuid,
-                                               const std::string &attribute,
+bool MetadataStorage::query_instance_attribute(std::string_view uuid,
+                                               std::string_view attribute,
                                                shcore::Value *out_value) const {
-  auto result = execute_sql(
-      shcore::sqlstring("SELECT attributes->'$." + attribute +
-                            "' FROM mysql_innodb_cluster_metadata.instances"
-                            " WHERE mysql_server_uuid=?",
-                        0)
-      << uuid);
+  auto stmt = shcore::str_format(
+      "SELECT attributes->'$.%.*s' FROM "
+      "mysql_innodb_cluster_metadata.instances WHERE mysql_server_uuid = ?",
+      static_cast<int>(attribute.length()), attribute.data());
+
+  auto result = execute_sql(shcore::sqlstring(stmt, 0) << uuid);
 
   if (auto row = result->fetch_one()) {
     if (!row->is_null(0)) {
@@ -938,31 +940,31 @@ bool MetadataStorage::query_instance_attribute(const std::string &uuid,
   return false;
 }
 
-void MetadataStorage::update_instance_attribute(const std::string &uuid,
-                                                const std::string &attribute,
+void MetadataStorage::update_instance_attribute(std::string_view uuid,
+                                                std::string_view attribute,
                                                 const shcore::Value &value) {
   if (value) {
-    shcore::sqlstring query(
-        "UPDATE mysql_innodb_cluster_metadata.instances"
-        " SET attributes = json_set(attributes, '$." +
-            attribute +
-            "', CAST(? as JSON))"
-            " WHERE mysql_server_uuid = ?",
-        0);
+    auto stmt = shcore::str_format(
+        "UPDATE mysql_innodb_cluster_metadata.instances SET attributes = "
+        "json_set(attributes, '$.%.*s', CAST(? as JSON)) WHERE "
+        "mysql_server_uuid = ?",
+        static_cast<int>(attribute.length()), attribute.data());
 
+    shcore::sqlstring query(stmt, 0);
     query << value.repr() << uuid;
     query.done();
+
     execute_sql(query);
   } else {
-    shcore::sqlstring query(
-        "UPDATE mysql_innodb_cluster_metadata.instances"
-        " SET attributes = json_remove(attributes, '$." +
-            attribute +
-            "')"
-            " WHERE mysql_server_uuid = ?",
-        0);
+    auto stmt = shcore::str_format(
+        "UPDATE mysql_innodb_cluster_metadata.instances SET attributes = "
+        "json_remove(attributes, '$.%.*s') WHERE mysql_server_uuid = ?",
+        static_cast<int>(attribute.length()), attribute.data());
+
+    shcore::sqlstring query(stmt, 0);
     query << uuid;
     query.done();
+
     execute_sql(query);
   }
 }
@@ -1069,9 +1071,15 @@ std::string repl_account_host_key(Cluster_type type) {
 void MetadataStorage::update_instance_repl_account(
     const std::string &instance_uuid, Cluster_type type,
     const std::string &recovery_account_user,
-    const std::string &recovery_account_host) {
+    const std::string &recovery_account_host, Transaction_undo *undo) {
   shcore::sqlstring query;
 
+  if (undo)
+    undo->add_snapshot_for_update(
+        "mysql_innodb_cluster_metadata.instances", "attributes",
+        *get_md_server(),
+        shcore::sqlformat("mysql_server_uuid = ?", instance_uuid));
+
   if (!recovery_account_user.empty()) {
     query = shcore::sqlstring(
         "UPDATE mysql_innodb_cluster_metadata.instances"
@@ -1126,9 +1134,14 @@ std::pair<std::string, std::string> MetadataStorage::get_instance_repl_account(
 
 void MetadataStorage::update_cluster_repl_account(
     const Cluster_id &cluster_id, const std::string &repl_account_user,
-    const std::string &repl_account_host) {
+    const std::string &repl_account_host, Transaction_undo *undo) {
   shcore::sqlstring query;
 
+  if (undo)
+    undo->add_snapshot_for_update(
+        "mysql_innodb_cluster_metadata.clusters", "attributes",
+        *get_md_server(), shcore::sqlformat("cluster_id = ?", cluster_id));
+
   if (!repl_account_user.empty()) {
     query = shcore::sqlstring(
         "UPDATE mysql_innodb_cluster_metadata.clusters"
@@ -1139,7 +1152,6 @@ void MetadataStorage::update_cluster_repl_account(
         0);
     query << repl_account_user;
     query << repl_account_host;
-
   } else {
     // if repl_account user is empty, clear existing recovery attributes
     // of the instance from the metadata.
@@ -1260,15 +1272,14 @@ std::string MetadataStorage::get_recovery_account_user(
   return row->get_string(0);
 }
 
-size_t MetadataStorage::iterate_recovery_account_mismatch(
+size_t MetadataStorage::iterate_recovery_account(
     const std::function<bool(uint32_t, std::string)> &cb) {
   auto query =
       "SELECT CAST(attributes->>'$.server_id' AS UNSIGNED), "
       "attributes->>'$.recoveryAccountUser' FROM "
       "mysql_innodb_cluster_metadata.instances WHERE "
       "(COALESCE(CAST(attributes->>'$.server_id' AS UNSIGNED), 0) > 0) AND "
-      "(attributes->>'$.recoveryAccountUser' <> "
-      "CONCAT(\"mysql_innodb_cluster_\", attributes->>'$.server_id'))"_sql;
+      "(attributes->>'$.recoveryAccountUser' IS NOT NULL)"_sql;
 
   auto result = execute_sql(query);
 
@@ -1291,7 +1302,8 @@ void MetadataStorage::remove_instance(const std::string &instance_address) {
   execute_sql(query);
 }
 
-void MetadataStorage::drop_cluster(const std::string &cluster_name) {
+void MetadataStorage::drop_cluster(const std::string &cluster_name,
+                                   Transaction_undo *undo) {
   auto get_cluster_id = [this](const std::string &full_cluster_name) {
     Cluster_id cluster_id;
     shcore::sqlstring query;
@@ -1329,23 +1341,27 @@ void MetadataStorage::drop_cluster(const std::string &cluster_name) {
     return cluster_id;
   };
 
-  Transaction tx(shared_from_this());
-
   // It exists, so let's get the cluster_id and move on
   Cluster_id cluster_id = get_cluster_id(cluster_name);
 
+  if (undo)
+    undo->add_snapshot_for_delete(
+        "mysql_innodb_cluster_metadata.instances", *get_md_server(),
+        shcore::sqlformat("cluster_id = ?", cluster_id));
   execute_sqlf(
       "DELETE FROM mysql_innodb_cluster_metadata.instances "
       "WHERE cluster_id = ?",
       cluster_id);
 
   // Remove the cluster
+  if (undo)
+    undo->add_snapshot_for_delete(
+        "mysql_innodb_cluster_metadata.clusters", *get_md_server(),
+        shcore::sqlformat("cluster_id = ?", cluster_id));
   execute_sqlf(
       "DELETE from mysql_innodb_cluster_metadata.clusters "
       "WHERE cluster_id = ?",
       cluster_id);
-
-  tx.commit();
 }
 
 void MetadataStorage::update_cluster_name(const Cluster_id &cluster_id,
@@ -1480,6 +1496,8 @@ Instance_metadata MetadataStorage::unserialize_instance(
   }
 
   instance.server_id = row.get_uint("server_id", 0);
+  if (row.has_field("cert_subject"))
+    instance.cert_subject = row.get_string("cert_subject", 0);
 
   if (row.has_field("hidden_from_router")) {
     instance.hidden_from_router = row.get_uint("hidden_from_router", 0);
@@ -1533,32 +1551,31 @@ void MetadataStorage::get_replica_set_primary_info(const Cluster_id &cluster_id,
 
 std::vector<Instance_metadata> MetadataStorage::get_all_instances(
     Cluster_id cluster_id) {
-  std::vector<Instance_metadata> ret_val;
+  if (real_version() == metadata::kNotInstalled) return {};
 
-  if (real_version() != metadata::kNotInstalled) {
-    std::string query(get_instance_query(m_real_md_version));
+  std::string query(get_instance_query(m_real_md_version));
 
-    // If a different schema is provided, uses it
-    if (m_md_version_schema != metadata::kMetadataSchemaName)
-      query = shcore::str_replace(query, metadata::kMetadataSchemaName,
-                                  m_md_version_schema);
+  // If a different schema is provided, uses it
+  if (m_md_version_schema != metadata::kMetadataSchemaName)
+    query = shcore::str_replace(query, metadata::kMetadataSchemaName,
+                                m_md_version_schema);
 
-    std::shared_ptr<mysqlshdk::db::IResult> result;
+  std::shared_ptr<mysqlshdk::db::IResult> result;
 
-    if (cluster_id.empty()) {
-      result = execute_sql(query);
+  if (cluster_id.empty()) {
+    result = execute_sql(query);
+  } else {
+    if (m_real_md_version.get_major() == 1) {
+      result = execute_sqlf(query + " WHERE r.cluster_id = ?",
+                            std::atoi(cluster_id.c_str()));
     } else {
-      if (m_real_md_version.get_major() == 1) {
-        result = execute_sqlf(query + " WHERE r.cluster_id = ?",
-                              std::atoi(cluster_id.c_str()));
-      } else {
-        result = execute_sqlf(query + " WHERE i.cluster_id = ?", cluster_id);
-      }
+      result = execute_sqlf(query + " WHERE i.cluster_id = ?", cluster_id);
     }
+  }
 
-    while (auto row = result->fetch_one_named()) {
-      ret_val.push_back(unserialize_instance(row, &m_real_md_version));
-    }
+  std::vector<Instance_metadata> ret_val;
+  while (auto row = result->fetch_one_named()) {
+    ret_val.push_back(unserialize_instance(row, &m_real_md_version));
   }
 
   return ret_val;
@@ -1684,7 +1701,7 @@ void MetadataStorage::get_lock_exclusive() const {
   // Try to acquire the specified lock.
   // NOTE: Only one lock per namespace is used because lock release is
   //       performed by namespace.
-  if (mysqlshdk::mysql::has_lock_service_udfs(*m_md_server)) {
+  if (mysqlshdk::mysql::has_lock_service(*m_md_server)) {
     // No need install Locking Service UDFs, already done at a higher level
     // (instance) if needed/supported. Thus, we only try to acquire the lock if
     // the lock UDFs are available (skipped otherwise).
@@ -1722,7 +1739,7 @@ void MetadataStorage::release_lock(bool no_throw) const {
   //       otherwise do nothing (ignore if concurrent execution is not
   //       supported, e.g., lock service plugin not available).
   try {
-    if (mysqlshdk::mysql::has_lock_service_udfs(*m_md_server)) {
+    if (mysqlshdk::mysql::has_lock_service(*m_md_server)) {
       log_debug("Releasing locks for '%s' on %s.", k_lock_name_metadata,
                 m_md_server->descr().c_str());
       mysqlshdk::mysql::release_lock(*m_md_server, k_lock_name_metadata);
@@ -1753,8 +1770,6 @@ constexpr const char *k_async_view_change_reason_remove_instance =
 
 Cluster_id MetadataStorage::create_async_cluster_record(
     Replica_set_impl *cluster, bool adopted) {
-  Transaction tx(shared_from_this());
-
   Cluster_id cluster_id;
   try {
     auto result = execute_sqlf("SELECT uuid()");
@@ -1794,8 +1809,6 @@ Cluster_id MetadataStorage::create_async_cluster_record(
 
   cluster->set_id(cluster_id);
 
-  tx.commit();
-
   return cluster_id;
 }
 
@@ -1887,8 +1900,6 @@ Instance_id MetadataStorage::record_async_member_added(
   // Always release locks at the end, when leaving the function scope.
   auto finally = shcore::on_leave_scope([this]() { release_lock(); });
 
-  Transaction tx(shared_from_this());
-
   Instance_id member_id = insert_instance(member);
 
   uint32_t aclvid;
@@ -1925,8 +1936,6 @@ Instance_id MetadataStorage::record_async_member_added(
       member.cluster_id, aclvid, member_id, member.master_id, member.master_id,
       member.primary_master, member_id);
 
-  tx.commit();
-
   return member_id;
 }
 
@@ -1938,8 +1947,6 @@ void MetadataStorage::record_async_member_rejoined(
   // Always release locks at the end, when leaving the function scope.
   auto finally = shcore::on_leave_scope([this]() { release_lock(); });
 
-  Transaction tx(shared_from_this());
-
   uint32_t aclvid;
   uint32_t last_aclvid;
 
@@ -1973,8 +1980,6 @@ void MetadataStorage::record_async_member_rejoined(
       " )",
       member.cluster_id, aclvid, member.id, member.master_id, member.master_id,
       member.primary_master, member.id);
-
-  tx.commit();
 }
 
 void MetadataStorage::record_async_member_removed(const Cluster_id &cluster_id,
@@ -1988,8 +1993,6 @@ void MetadataStorage::record_async_member_removed(const Cluster_id &cluster_id,
   uint32_t aclvid;
   uint32_t last_aclvid;
 
-  Transaction tx(shared_from_this());
-
   begin_acl_change_record(cluster_id,
                           k_async_view_change_reason_remove_instance, &aclvid,
                           &last_aclvid);
@@ -2010,8 +2013,6 @@ void MetadataStorage::record_async_member_removed(const Cluster_id &cluster_id,
       "DELETE FROM mysql_innodb_cluster_metadata.instances "
       "WHERE instance_id = ?",
       instance_id);
-
-  tx.commit();
 }
 
 void MetadataStorage::record_async_primary_switch(Instance_id new_primary_id) {
@@ -2024,8 +2025,6 @@ void MetadataStorage::record_async_primary_switch(Instance_id new_primary_id) {
   uint32_t aclvid;
   uint32_t last_aclvid;
 
-  Transaction tx(shared_from_this());
-
   auto res = execute_sqlf(
       "SELECT c.cluster_id, c.async_topology_type"
       " FROM mysql_innodb_cluster_metadata.v2_ar_clusters c"
@@ -2101,8 +2100,6 @@ void MetadataStorage::record_async_primary_switch(Instance_id new_primary_id) {
         assert(0);
         throw std::logic_error("Internal error");
     }
-
-    tx.commit();
   } else {
     log_error("Async cluster for instance %i has metadata errors",
               new_primary_id);
@@ -2120,8 +2117,6 @@ void MetadataStorage::record_async_primary_forced_switch(
   uint32_t aclvid;
   uint32_t last_aclvid;
 
-  Transaction tx(shared_from_this());
-
   auto res = execute_sqlf(
       "SELECT c.cluster_id, c.async_topology_type"
       " FROM mysql_innodb_cluster_metadata.v2_ar_clusters c"
@@ -2193,8 +2188,6 @@ void MetadataStorage::record_async_primary_forced_switch(
         assert(0);
         throw std::logic_error("Internal error");
     }
-
-    tx.commit();
   } else {
     log_error("Async cluster for instance %i has metadata errors",
               new_primary_id);
@@ -2361,58 +2354,45 @@ static void parse_router_definition(const std::string &router_def,
   }
 }
 
-bool MetadataStorage::remove_router(const std::string &router_def) {
+bool MetadataStorage::remove_router(const std::string &router_def,
+                                    bool lock_metadata) {
   std::string address;
   std::string name;
-
   parse_router_definition(router_def, &address, &name);
 
-  // Acquire required lock on the Metadata (try at most during 60 sec).
-  get_lock_exclusive();
+  shcore::on_leave_scope finally;
+  if (lock_metadata) {
+    // Acquire required lock on the Metadata (try at most during 60 sec).
+    get_lock_exclusive();
 
-  // Always release locks at the end, when leaving the function scope.
-  auto finally = shcore::on_leave_scope([this]() { release_lock(); });
+    // Always release locks at the end, when leaving the function scope.
+    finally = shcore::on_leave_scope([this]() { release_lock(); });
+  }
 
   // This has to support MD versions 1.0 and 2.0, so that we can remove older
   // router versions while upgrading the MD.
+  std::shared_ptr<mysqlshdk::db::IResult> result;
   if (real_version().get_major() == 1) {
-    auto result = execute_sqlf(
-        "SELECT router_id FROM mysql_innodb_cluster_metadata.routers r"
-        " JOIN mysql_innodb_cluster_metadata.hosts h ON r.host_id = h.host_id"
-        " WHERE r.router_name = ? AND LOWER(h.host_name) = LOWER(?)",
+    result = execute_sqlf(
+        "SELECT router_id FROM mysql_innodb_cluster_metadata.routers r JOIN "
+        "mysql_innodb_cluster_metadata.hosts h ON r.host_id = h.host_id WHERE "
+        "r.router_name = ? AND LOWER(h.host_name) = LOWER(?)",
         name, address);
-
-    if (auto row = result->fetch_one()) {
-      int router_id = row->get_int(0);
-
-      execute_sqlf(
-          "DELETE FROM mysql_innodb_cluster_metadata.routers"
-          " WHERE router_id = ?",
-          router_id);
-
-      return true;
-    } else {
-      return false;
-    }
   } else {
-    auto result = execute_sqlf(
-        "SELECT router_id FROM mysql_innodb_cluster_metadata.routers r"
-        " WHERE r.router_name = ? AND r.address = ?",
+    result = execute_sqlf(
+        "SELECT router_id FROM mysql_innodb_cluster_metadata.routers r WHERE "
+        "r.router_name = ? AND r.address = ?",
         name, address);
-
-    if (auto row = result->fetch_one()) {
-      int router_id = row->get_int(0);
-
-      execute_sqlf(
-          "DELETE FROM mysql_innodb_cluster_metadata.routers"
-          " WHERE router_id = ?",
-          router_id);
-
-      return true;
-    } else {
-      return false;
-    }
   }
+
+  auto row = result->fetch_one();
+  if (!row) return false;
+
+  execute_sqlf(
+      "DELETE FROM mysql_innodb_cluster_metadata.routers WHERE router_id = ?",
+      row->get_int(0));
+
+  return true;
 }
 
 void MetadataStorage::set_target_cluster_for_all_routers(
@@ -2673,13 +2653,6 @@ void MetadataStorage::cleanup_for_cluster(Cluster_id cluster_id) {
 Cluster_set_id MetadataStorage::create_cluster_set_record(
     Cluster_set_impl *clusterset, Cluster_id seed_cluster_id,
     shcore::Dictionary_t seed_attributes) {
-  get_lock_exclusive();
-
-  // Always release locks at the end, when leaving the function scope.
-  auto finally = shcore::on_leave_scope([this]() { release_lock(); });
-
-  Transaction tx(shared_from_this());
-
   execute_sqlf(
       "CALL mysql_innodb_cluster_metadata.v2_cs_created(?, ?, ?, @_cs_id)",
       clusterset->get_name(), seed_cluster_id,
@@ -2689,37 +2662,19 @@ Cluster_set_id MetadataStorage::create_cluster_set_record(
       execute_sqlf("select @_cs_id")->fetch_one()->get_string(0, "");
   clusterset->set_id(cs_id);
 
-  tx.commit();
-
   return cs_id;
 }
 
 void MetadataStorage::record_cluster_set_member_added(
     const Cluster_set_member_metadata &cluster) {
-  // Acquire required lock on the Metadata (try at most during 60 sec).
-  get_lock_exclusive();
-
-  // Always release locks at the end, when leaving the function scope.
-  auto finally = shcore::on_leave_scope([this]() { release_lock(); });
-
-  Transaction tx(shared_from_this());
-
   execute_sqlf(
       "CALL mysql_innodb_cluster_metadata.v2_cs_member_added(?, ?, ?, '{}')",
       cluster.cluster_set_id, cluster.cluster.cluster_id,
       cluster.master_cluster_id);
-
-  tx.commit();
 }
 
 void MetadataStorage::record_cluster_set_member_removed(
     const Cluster_set_id &cs_id, const Cluster_id &cluster_id) {
-  // Acquire required lock on the Metadata (try at most during 60 sec).
-  get_lock_exclusive();
-
-  // Always release locks at the end, when leaving the function scope.
-  auto finally = shcore::on_leave_scope([this]() { release_lock(); });
-
   execute_sqlf("CALL mysql_innodb_cluster_metadata.v2_cs_member_removed(?, ?)",
                cs_id, cluster_id);
 }
@@ -2727,26 +2682,14 @@ void MetadataStorage::record_cluster_set_member_removed(
 void MetadataStorage::record_cluster_set_member_rejoined(
     const Cluster_set_id &cs_id, const Cluster_id &cluster_id,
     const Cluster_id &master_cluster_id) {
-  // Acquire required lock on the Metadata (try at most during 60 sec).
-  get_lock_exclusive();
-
-  // Always release locks at the end, when leaving the function scope.
-  auto finally = shcore::on_leave_scope([this]() { release_lock(); });
-
-  Transaction tx(shared_from_this());
-
   execute_sqlf(
       "CALL mysql_innodb_cluster_metadata.v2_cs_member_rejoined(?, ?, ?, '{}')",
       cs_id, cluster_id, master_cluster_id);
-
-  tx.commit();
 }
 
 void MetadataStorage::record_cluster_set_primary_switch(
     const Cluster_set_id &cs_id, const Cluster_id &new_primary_id,
     const std::list<Cluster_id> &invalidated) {
-  Transaction tx(shared_from_this());
-
   execute_sqlf(
       "CALL mysql_innodb_cluster_metadata.v2_cs_primary_changed(?, ?, "
       "'{}')",
@@ -2757,15 +2700,11 @@ void MetadataStorage::record_cluster_set_primary_switch(
         "CALL mysql_innodb_cluster_metadata.v2_cs_add_invalidated_member(?, ?)",
         cs_id, c);
   }
-
-  tx.commit();
 }
 
 void MetadataStorage::record_cluster_set_primary_failover(
     const Cluster_set_id &cs_id, const Cluster_id &cluster_id,
     const std::list<Cluster_id> &invalidated) {
-  Transaction tx(shared_from_this());
-
   execute_sqlf(
       "CALL mysql_innodb_cluster_metadata.v2_cs_primary_force_changed(?, ?, "
       "'{}')",
@@ -2776,8 +2715,6 @@ void MetadataStorage::record_cluster_set_primary_failover(
         "CALL mysql_innodb_cluster_metadata.v2_cs_add_invalidated_member(?, ?)",
         cs_id, c);
   }
-
-  tx.commit();
 }
 
 bool MetadataStorage::check_metadata(mysqlshdk::utils::Version *out_version,
