@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -38,6 +38,7 @@
 #include "modules/adminapi/common/metadata_management_mysql.h"
 #include "mysqlshdk/libs/db/session.h"
 #include "mysqlshdk/libs/mysql/group_replication.h"
+#include "mysqlshdk/libs/mysql/undo.h"
 #include "mysqlshdk/libs/mysql/utils.h"
 
 namespace mysqlsh {
@@ -60,6 +61,7 @@ struct Instance_metadata {
   std::string grendpoint;
   uint32_t server_id = 0;
   bool hidden_from_router = false;
+  std::string cert_subject;
 
   // GR clusters only
   std::string group_name;
@@ -133,6 +135,8 @@ class Cluster_impl;
 class Cluster_set_impl;
 class Replica_set_impl;
 
+using mysqlshdk::mysql::Transaction_undo;
+
 inline uint32_t cluster_set_view_id_generation(uint64_t id) {
   return (id >> 32);
 }
@@ -142,7 +146,7 @@ inline uint32_t cluster_set_view_id_generation(uint64_t id) {
  * Represents a session to a Metadata Storage
  */
 #endif
-class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
+class MetadataStorage {
  protected:
   MetadataStorage() = default;
 
@@ -212,19 +216,20 @@ class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
   Instance_id insert_instance(const Instance_metadata &instance);
   void update_instance(const Instance_metadata &instance);
   void remove_instance(const std::string &instance_address);
-  void drop_cluster(const std::string &cluster_name);
+  void drop_cluster(const std::string &cluster_name,
+                    Transaction_undo *undo = nullptr);
 
   void update_cluster_name(const Cluster_id &cluster_id,
                            const std::string &new_cluster_name);
 
   bool query_cluster_attribute(const Cluster_id &cluster_id,
-                               const std::string &attribute,
+                               std::string_view attribute,
                                shcore::Value *out_value) const;
 
   void update_cluster_attribute(const Cluster_id &cluster_id,
-                                const std::string &attribute,
+                                std::string_view attribute,
                                 const shcore::Value &value);
-  void update_clusters_attribute(const std::string &attribute,
+  void update_clusters_attribute(std::string_view attribute,
                                  const shcore::Value &value);
 
   void update_cluster_capability(
@@ -237,19 +242,19 @@ class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
                                 shcore::Value *out_value) const;
 
   void update_cluster_set_attribute(const Cluster_set_id &clusterset_id,
-                                    const std::string &attribute,
+                                    std::string_view attribute,
                                     const shcore::Value &value);
 
   bool query_cluster_set_attribute(const Cluster_set_id &clusterset_id,
-                                   const std::string &attribute,
+                                   std::string_view attribute,
                                    shcore::Value *out_value) const;
 
-  bool query_instance_attribute(const std::string &uuid,
-                                const std::string &attribute,
+  bool query_instance_attribute(std::string_view uuid,
+                                std::string_view attribute,
                                 shcore::Value *out_value) const;
 
-  void update_instance_attribute(const std::string &uuid,
-                                 const std::string &attribute,
+  void update_instance_attribute(std::string_view uuid,
+                                 std::string_view attribute,
                                  const shcore::Value &value);
 
   /**
@@ -306,7 +311,8 @@ class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
   void update_instance_repl_account(const std::string &instance_uuid,
                                     Cluster_type type,
                                     const std::string &recovery_account_user,
-                                    const std::string &recovery_account_host);
+                                    const std::string &recovery_account_host,
+                                    Transaction_undo *undo = nullptr);
   /**
    * Fetch from the metadata the recovery account being used by the instance
    * with the given uuid.
@@ -320,7 +326,8 @@ class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
 
   void update_cluster_repl_account(const Cluster_id &cluster_id,
                                    const std::string &repl_account_user,
-                                   const std::string &repl_account_host);
+                                   const std::string &repl_account_host,
+                                   Transaction_undo *undo = nullptr);
 
   std::pair<std::string, std::string> get_cluster_repl_account(
       const Cluster_id &cluster_id) const;
@@ -343,7 +350,7 @@ class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
   std::string get_recovery_account_user(const Cluster_id &cluster_id,
                                         const std::string &address);
 
-  size_t iterate_recovery_account_mismatch(
+  size_t iterate_recovery_account(
       const std::function<bool(uint32_t, std::string)> &cb);
 
   virtual bool is_instance_on_cluster(const Cluster_id &cluster_id,
@@ -455,9 +462,10 @@ class MetadataStorage : public std::enable_shared_from_this<MetadataStorage> {
    * Deletes metadata for the named router instance.
    *
    * @param router_def router identifier, as address[::name]
+   * @param lock_metadata controls if the metadata is locked for the operation
    * @return false if router_def doesn't match any router instances
    */
-  bool remove_router(const std::string &router_def);
+  bool remove_router(const std::string &router_def, bool lock_metadata = false);
 
   /**
    * Sets a target-cluster for all Routers of a Cluster
