@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2022, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 #include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/preconditions.h"
 #include "modules/adminapi/common/provision.h"
+#include "modules/adminapi/common/server_features.h"
 #include "modules/adminapi/common/sql.h"
 #include "modules/adminapi/common/validations.h"
 #include "modules/adminapi/mod_dba.h"
@@ -576,8 +577,7 @@ void Reboot_cluster_from_complete_outage::resolve_local_address(
   } else {
     // The default value for communicationStack must be 'mysql' if the target
     // instance is running 8.0.27+
-    if (m_target_instance->get_version() >=
-        k_mysql_communication_stack_initial_version) {
+    if (supports_mysql_communication_stack(m_target_instance->get_version())) {
       communication_stack = kCommunicationStackMySQL;
     } else {
       communication_stack = kCommunicationStackXCom;
@@ -605,7 +605,8 @@ void Reboot_cluster_from_complete_outage::check_instance_configuration() {
   // Cluster.rescan() and the target instance wasn't restarted so the value is
   // not effective yet. Attempt to read it and if not empty, set it right away
   // in the GR options to be used
-  if (m_target_instance->get_version() >= k_min_cs_version) {
+  if (m_target_instance->get_version() >=
+      Precondition_checker::k_min_cs_version) {
     std::string view_change_uuid_persisted =
         m_target_instance
             ->get_persisted_value("group_replication_view_change_uuid")
@@ -678,7 +679,7 @@ void Reboot_cluster_from_complete_outage::reboot_seed(
         if (status == mysqlshdk::mysql::Replication_channel::OFF) {
           try {
             mysqlshdk::mysql::reset_slave(
-                m_target_instance.get(), k_clusterset_async_channel_name, true);
+                *m_target_instance, k_clusterset_async_channel_name, true);
           } catch (const shcore::Error &e) {
             throw shcore::Exception::mysql_error_with_code(e.what(), e.code());
           }
@@ -829,19 +830,30 @@ void Reboot_cluster_from_complete_outage::reboot_seed(
       }
 
       // Create a new recovery account
-      repl_account = mysqlshdk::gr::create_recovery_user(
-          repl_account.user, *m_target_instance, {repl_account_host}, {}, true,
-          false, true);
+      {
+        std::vector<std::string> hosts;
+        hosts.push_back(repl_account_host);
+
+        mysqlshdk::gr::Create_recovery_user_options options;
+        options.clone_supported = true;
+        options.auto_failover = false;
+        options.mysql_comm_stack_supported = true;
+
+        repl_account = mysqlshdk::gr::create_recovery_user(
+            repl_account.user, *m_target_instance, hosts, options);
+      }
 
       // Change GR's recovery replication credentials in all possible
       // donors so whenever GR picks a suitable donor it will be able to
       // connect and authenticate at the target
       // NOTE: Instances in RECOVERING must be skipped since won't be used
       // as donor and the change source command would fail anyway
+      mysqlshdk::mysql::Replication_credentials_options options;
+      options.password = repl_account.password.value_or("");
+
       mysqlshdk::mysql::change_replication_credentials(
-          *m_target_instance, repl_account.user,
-          repl_account.password.value_or(""),
-          mysqlshdk::gr::k_gr_recovery_channel);
+          *m_target_instance, mysqlshdk::gr::k_gr_recovery_channel,
+          repl_account.user, options);
 
       if (m_cluster->impl()->is_cluster_set_member() &&
           !m_cluster->impl()->is_primary_cluster()) {
@@ -888,8 +900,8 @@ void Reboot_cluster_from_complete_outage::reboot_seed(
         transaction_size_limit = value.as_int();
       } else {
         // Use what's set in the instance
-        transaction_size_limit =
-            m_cluster->impl()->get_transaction_size_limit();
+        transaction_size_limit = get_transaction_size_limit(
+            *m_cluster->impl()->get_cluster_server());
       }
 
       log_info("Using Group Replication transaction size limit: %" PRId64,
@@ -898,6 +910,20 @@ void Reboot_cluster_from_complete_outage::reboot_seed(
       m_options.gr_options.transaction_size_limit = transaction_size_limit;
     }
 
+    // Get the persisted value of paxosSingleLeader to use it
+    if (!m_options.gr_options.paxos_single_leader.has_value() &&
+        m_target_instance->is_set_persist_supported()) {
+      std::string paxos_single_leader =
+          m_target_instance
+              ->get_persisted_value("group_replication_paxos_single_leader")
+              .value_or("");
+
+      if (!paxos_single_leader.empty()) {
+        m_options.gr_options.paxos_single_leader =
+            shcore::str_caseeq(paxos_single_leader, "on") ? true : false;
+      }
+    }
+
     log_info("Starting cluster with '%s' using account %s",
              m_target_instance->descr().c_str(),
              m_target_instance->get_connection_options().get_user().c_str());
@@ -906,9 +932,21 @@ void Reboot_cluster_from_complete_outage::reboot_seed(
     auto multi_primary = m_cluster->impl()->get_cluster_topology_type() ==
                          mysqlshdk::gr::Topology_mode::MULTI_PRIMARY;
 
+    bool requires_certificates{false};
+    switch (m_cluster->impl()->query_cluster_auth_type()) {
+      case mysqlsh::dba::Replication_auth_type::CERT_ISSUER:
+      case mysqlsh::dba::Replication_auth_type::CERT_SUBJECT:
+      case mysqlsh::dba::Replication_auth_type::CERT_ISSUER_PASSWORD:
+      case mysqlsh::dba::Replication_auth_type::CERT_SUBJECT_PASSWORD:
+        requires_certificates = true;
+      default:
+        break;
+    }
+
     // Start the cluster to bootstrap Group Replication.
     mysqlsh::dba::start_cluster(*m_target_instance, m_options.gr_options,
-                                multi_primary, cfg.get());
+                                requires_certificates, multi_primary,
+                                cfg.get());
 
     // Wait for the seed instance to become ONLINE in the Group.
     // Especially relevant on Replica Clusters to ensure the seed instance is
@@ -1158,6 +1196,22 @@ std::shared_ptr<Cluster> Reboot_cluster_from_complete_outage::do_run() {
 
   // everything is ready, proceed with the reboot...
 
+  // will have to lock the target instance and all reachable members
+  mysqlshdk::mysql::Lock_scoped_list i_locks;
+  {
+    auto new_seed = best_instance_gtid ? best_instance_gtid : m_target_instance;
+    i_locks.push_back(new_seed->get_lock_exclusive());
+
+    instances.push_back(m_target_instance);  // temporary
+
+    for (const auto &instance : instances) {
+      if (instance->get_uuid() != new_seed->get_uuid())
+        i_locks.push_back(instance->get_lock_exclusive());
+    }
+
+    instances.pop_back();
+  }
+
   // reboot seed
   if (best_instance_gtid) {
     // if we have a better instance in the cluster (in terms of highest GTID),
