@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -170,24 +170,12 @@ String Cluster::getName() {}
 str Cluster::get_name() {}
 #endif
 
-shcore::Value Cluster::call(const std::string &name,
-                            const shcore::Argument_list &args) {
-  // Throw an error if the cluster has already been dissolved
-  assert_valid(name);
-  return Cpp_object_bridge::call(name, args);
-}
-
 shcore::Value Cluster::get_member(const std::string &prop) const {
-  shcore::Value ret_val;
-
   // Throw an error if the cluster has already been dissolved
   assert_valid(prop);
 
-  if (prop == "name")
-    ret_val = shcore::Value(impl()->get_name());
-  else
-    ret_val = shcore::Cpp_object_bridge::get_member(prop);
-  return ret_val;
+  if (prop == "name") return shcore::Value(impl()->get_name());
+  return shcore::Cpp_object_bridge::get_member(prop);
 }
 
 void Cluster::assert_valid(const std::string &option_name) const {
@@ -201,12 +189,12 @@ void Cluster::assert_valid(const std::string &option_name) const {
       name = get_function_name(option_name, false);
       throw shcore::Exception::runtime_error(class_name() + "." + name + ": " +
                                              "Can't call function '" + name +
-                                             "' on an offline cluster");
+                                             "' on an offline Cluster");
     } else {
       name = get_member_name(option_name, shcore::current_naming_style());
       throw shcore::Exception::runtime_error(class_name() + "." + name + ": " +
                                              "Can't access object member '" +
-                                             name + "' on an offline cluster");
+                                             name + "' on an offline Cluster");
     }
   }
 
@@ -267,6 +255,7 @@ ${OPT_INTERACTIVE}
 ${CLUSTER_OPT_EXIT_STATE_ACTION}
 ${CLUSTER_OPT_MEMBER_WEIGHT}
 ${CLUSTER_OPT_AUTO_REJOIN_TRIES}
+${OPT_CERT_SUBJECT}
 
 The password may be contained on the instance definition, however, it can be
 overwritten if it is specified on the options.
@@ -337,6 +326,9 @@ None Cluster::add_instance(InstanceDef instance, dict options) {}
 void Cluster::add_instance(
     const Connection_options &instance_def_,
     const shcore::Option_pack_ref<cluster::Add_instance_options> &options) {
+  // Throw an error if the cluster has already been dissolved
+  assert_valid("addInstance");
+
   auto instance_def = instance_def_;
 
   if (!options->password.is_null()) {
@@ -344,11 +336,11 @@ void Cluster::add_instance(
   }
 
   // Validate the label value.
-  if (!options->label.is_null()) {
-    mysqlsh::dba::validate_label(*(options->label));
+  if (options->label.has_value()) {
+    mysqlsh::dba::validate_label(*options->label);
 
     if (!impl()->get_metadata_storage()->is_instance_label_unique(
-            impl()->get_id(), *(options->label))) {
+            impl()->get_id(), *options->label)) {
       throw shcore::Exception::argument_error(
           "An instance with label '" + *(options->label) +
           "' is already part of this InnoDB cluster");
@@ -481,6 +473,9 @@ None Cluster::remove_instance(InstanceDef instance, dict options) {}
 void Cluster::remove_instance(
     const Connection_options &instance_def_,
     const shcore::Option_pack_ref<cluster::Remove_instance_options> &options) {
+  // Throw an error if the cluster has already been dissolved
+  assert_valid("removeInstance");
+
   auto instance_def = instance_def_;
 
   if (!options->password.is_null()) {
@@ -1309,25 +1304,24 @@ username[@@host] where the host part is optional and if not provided defaults to
 The options dictionary may contain the following attributes:
 
 @li password: The password for the InnoDB cluster administrator account.
-@li dryRun: boolean value used to enable a dry run of the account setup
-process. Default value is False.
+${OPT_SETUP_ACCOUNT_OPTIONS_PASSWORD_EXPIRATION}
+${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_ISSUER}
+${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_SUBJECT}
+${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN}
 ${OPT_INTERACTIVE}
-@li update: boolean value that must be enabled to allow updating the privileges
-and/or password of existing accounts. Default value is False.
+${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE}
 
-If the user account does not exist, the password is mandatory.
+If the user account does not exist, either the password, requireCertIssuer or
+requireCertSubject are mandatory.
 
 If the user account exists, the update option must be enabled.
 
-If dryRun is used, the function will display information about the permissions
-to be granted to `user` account without actually creating and/or performing any
-changes on it.
+${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN_DETAIL}
 
 The interactive option can be used to explicitly enable or disable the
 interactive prompts that help the user through the account setup process.
 
-The update option must be enabled to allow updating an existing account's
-privileges and/or password.
+${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE_DETAIL}
 )*");
 
 /**
@@ -1378,25 +1372,23 @@ username[@@host] where the host part is optional and if not provided defaults to
 The options dictionary may contain the following attributes:
 
 @li password: The password for the MySQL Router account.
-@li dryRun: boolean value used to enable a dry run of the account setup
-process. Default value is False.
+${OPT_SETUP_ACCOUNT_OPTIONS_PASSWORD_EXPIRATION}
+${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_ISSUER}
+${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_SUBJECT}
+${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN}
 ${OPT_INTERACTIVE}
-@li update: boolean value that must be enabled to allow updating the privileges
-and/or password of existing accounts. Default value is False.
+${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE}
 
-If the user account does not exist, the password is mandatory.
+If the user account does not exist, either the password, requireCertIssuer or
+requireCertSubject are mandatory.
 
 If the user account exists, the update option must be enabled.
 
-If dryRun is used, the function will display information about the permissions
-to be granted to `user` account without actually creating and/or performing any
-changes on it.
+${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN_DETAIL}
 
-The interactive option can be used to explicitly enable or disable the
-interactive prompts that help the user through the account setup process.
+${OPT_SETUP_ACCOUNT_OPTIONS_INTERACTIVE_DETAIL}
 
-The update option must be enabled to allow updating an existing account's
-privileges and/or password.
+${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE_DETAIL}
 )*");
 
 /**
@@ -1588,14 +1580,13 @@ any other member using accounts with this hostname value.
 
 The clusterSetReplicationSslMode option supports the following values:
 
-@li REQUIRED: if used, SSL (encryption) will be enabled for the ClusterSet
-replication channels.
-@li DISABLED: if used, SSL (encryption) will be disabled for the ClusterSet
-replication channels.
-@li AUTO: if used, SSL (encryption) will be enabled if supported by the
-instance, otherwise disabled.
+@li DISABLED: TLS encryption is disabled for the ClusterSet replication channels.
+@li REQUIRED: TLS encryption is enabled for the ClusterSet replication channels.
+@li VERIFY_CA: like REQUIRED, but additionally verify the peer server TLS certificate against the configured Certificate Authority (CA) certificates.
+@li VERIFY_IDENTITY: like VERIFY_CA, but additionally verify that the peer server certificate matches the host to which the connection is attempted.
+@li AUTO: TLS encryption will be enabled if supported by the instance, otherwise disabled.
 
-If clusterSetReplicationSslMode is not specified AUTO will be used by default.
+If clusterSetReplicationSslMode is not specified, it defaults to the value of the cluster's memberSslMode option.
 )*");
 
 /**
@@ -1612,6 +1603,9 @@ shcore::Value Cluster::create_cluster_set(
     const std::string &domain_name,
     const shcore::Option_pack_ref<clusterset::Create_cluster_set_options>
         &options) {
+  // Throw an error if the cluster has already been dissolved
+  assert_valid("createClusterSet");
+
   return execute_with_pool(
       [&]() { return impl()->create_cluster_set(domain_name, *options); },
       false);
