@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2020, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -280,12 +280,10 @@ struct Instance_cache_builder::Query_helper {
 
 Instance_cache_builder::Instance_cache_builder(
     const std::shared_ptr<mysqlshdk::db::ISession> &session,
-    const Filter &included_schemas, const Object_filters &included_tables,
-    const Filter &excluded_schemas, const Object_filters &excluded_tables,
-    Instance_cache &&cache)
-    : m_session(session), m_cache(std::move(cache)) {
-  set_schema_filter(included_schemas, excluded_schemas);
-  set_table_filter(included_tables, excluded_tables);
+    const common::Filtering_options &filters, Instance_cache &&cache)
+    : m_session(session), m_cache(std::move(cache)), m_filters(filters) {
+  set_schema_filter();
+  set_table_filter();
 
   for (const auto &schema : m_cache.schemas) {
     if (!schema.second.tables.empty()) {
@@ -311,19 +309,19 @@ Instance_cache_builder::Instance_cache_builder(
 }
 
 Instance_cache_builder &Instance_cache_builder::metadata(
-    const Subobject_filters &partitions) {
+    const Partition_filters &partitions) {
   fetch_metadata(partitions);
   return *this;
 }
 
-Instance_cache_builder &Instance_cache_builder::users(const Users &included,
-                                                      const Users &excluded) {
+Instance_cache_builder &Instance_cache_builder::users() {
   Profiler profiler{"fetching users"};
 
   Schema_dumper sd{m_session};
 
-  m_cache.users = sd.get_users(included, excluded);
-  m_cache.roles = sd.get_roles(included, excluded);
+  const auto &users = m_filters.users();
+  m_cache.users = sd.get_users(users);
+  m_cache.roles = sd.get_roles(users);
 
   m_cache.filtered.users = m_cache.users.size();
   m_cache.total.users = count("user_privileges", {}, "DISTINCT grantee");
@@ -331,8 +329,7 @@ Instance_cache_builder &Instance_cache_builder::users(const Users &included,
   return *this;
 }
 
-Instance_cache_builder &Instance_cache_builder::events(
-    const Object_filters &included, const Object_filters &excluded) {
+Instance_cache_builder &Instance_cache_builder::events() {
   Profiler profiler{"fetching events"};
 
   Iterate_schema info;
@@ -342,7 +339,8 @@ Instance_cache_builder &Instance_cache_builder::events(
   };
   info.table_name = "events";
   // event names are case insensitive
-  info.where = object_filter(info, included, excluded);
+  info.where = object_filter(info, m_filters.events().included(),
+                             m_filters.events().excluded());
 
   iterate_schemas(info,
                   [this](const std::string &, Instance_cache::Schema *schema,
@@ -358,8 +356,7 @@ Instance_cache_builder &Instance_cache_builder::events(
   return *this;
 }
 
-Instance_cache_builder &Instance_cache_builder::routines(
-    const Object_filters &included, const Object_filters &excluded) {
+Instance_cache_builder &Instance_cache_builder::routines() {
   Profiler profiler{"fetching routines"};
 
   Iterate_schema info;
@@ -370,7 +367,8 @@ Instance_cache_builder &Instance_cache_builder::routines(
   };
   info.table_name = "routines";
   // routine names are case insensitive
-  info.where = object_filter(info, included, excluded);
+  info.where = object_filter(info, m_filters.routines().included(),
+                             m_filters.routines().excluded());
 
   const std::string procedure = "PROCEDURE";
 
@@ -392,8 +390,7 @@ Instance_cache_builder &Instance_cache_builder::routines(
   return *this;
 }
 
-Instance_cache_builder &Instance_cache_builder::triggers(
-    const Subobject_filters &included, const Subobject_filters &excluded) {
+Instance_cache_builder &Instance_cache_builder::triggers() {
   Profiler profiler{"fetching triggers"};
 
   if (has_tables()) {
@@ -406,7 +403,8 @@ Instance_cache_builder &Instance_cache_builder::triggers(
     };
     info.table_name = "triggers";
     // trigger names are case sensitive
-    info.where = trigger_filter(info, included, excluded);
+    info.where = trigger_filter(info, m_filters.triggers().included(),
+                                m_filters.triggers().excluded());
 
     // schema -> table -> triggers
     std::unordered_map<
@@ -508,27 +506,31 @@ void Instance_cache_builder::filter_tables() {
   iterate_schemas(
       info, [this](const std::string &, Instance_cache::Schema *schema,
                    const mysqlshdk::db::IRow *row) {
-        const auto table_name = row->get_string(1);  // TABLE_NAME
         const auto table_type = row->get_string(2);  // TABLE_TYPE
 
-        if ("BASE TABLE" == table_type) {
-          auto &table = schema->tables[table_name];
-          table.row_count = row->get_uint(3, 0);           // TABLE_ROWS
-          table.average_row_length = row->get_uint(4, 0);  // AVG_ROW_LENGTH
-          table.engine = row->get_string(5, "");           // ENGINE
-          table.create_options = row->get_string(6, "");   // CREATE_OPTIONS
-          table.comment = row->get_string(7, "");          // TABLE_COMMENT
+        if ("SYSTEM VIEW" == table_type) {
+          return;
+        }
 
+        const auto table_name = row->get_string(1);  // TABLE_NAME
+        const auto is_table = "BASE TABLE" == table_type;
+        Instance_cache::Table &target =
+            is_table ? schema->tables[table_name] : schema->views[table_name];
+
+        target.row_count = row->get_uint(3, 0);           // TABLE_ROWS
+        target.average_row_length = row->get_uint(4, 0);  // AVG_ROW_LENGTH
+        target.engine = row->get_string(5, "");           // ENGINE
+        target.create_options = row->get_string(6, "");   // CREATE_OPTIONS
+        target.comment = row->get_string(7, "");          // TABLE_COMMENT
+
+        if (is_table) {
           set_has_tables();
 
           ++m_cache.filtered.tables;
 
           DBUG_EXECUTE_IF("dumper_average_row_length_0",
-                          { table.average_row_length = 0; });
-        } else if ("VIEW" == table_type) {
-          // nop, just to insert the view
-          schema->views[table_name].character_set_client.clear();
-
+                          { target.average_row_length = 0; });
+        } else {
           set_has_views();
 
           ++m_cache.filtered.views;
@@ -541,7 +543,7 @@ void Instance_cache_builder::filter_tables() {
 }
 
 void Instance_cache_builder::fetch_metadata(
-    const Subobject_filters &partitions) {
+    const Partition_filters &partitions) {
   Profiler profiler{"fetching metadata"};
 
   fetch_ndbinfo();
@@ -583,7 +585,12 @@ void Instance_cache_builder::fetch_server_metadata() {
 
   m_cache.hostname = mysqlshdk::utils::Net::get_hostname();
 
-  m_cache.gtid_executed = Schema_dumper{m_session}.gtid_executed();
+  Schema_dumper dumper{m_session};
+  m_cache.gtid_executed = dumper.gtid_executed();
+
+  if (m_cache.server_version.version >= mysqlshdk::utils::Version(8, 0, 16)) {
+    m_cache.partial_revokes = dumper.partial_revokes();
+  }
 }
 
 void Instance_cache_builder::fetch_ndbinfo() {
@@ -648,45 +655,47 @@ void Instance_cache_builder::fetch_columns() {
       table_columns;
   // schema -> view -> columns
   std::unordered_map<
-      std::string,
-      std::unordered_map<std::string, std::map<uint64_t, std::string>>>
+      std::string, std::unordered_map<
+                       std::string, std::map<uint64_t, Instance_cache::Column>>>
       view_columns;
 
+  const auto create_column = [](const mysqlshdk::db::IRow *row) {
+    Instance_cache::Column column;
+    // these can be NULL in 8.0, as per output of 'SHOW COLUMNS', but it's
+    // not likely, as they're NOT NULL in definition of mysql.columns hidden
+    // table
+    column.name = row->get_string(2, "");  // COLUMN_NAME
+    column.quoted_name = shcore::quote_identifier(column.name);
+    const auto data_type = row->get_string(3, "");  // DATA_TYPE
+    column.type = mysqlshdk::db::dbstring_to_type(
+        data_type, row->get_string(6));  // COLUMN_TYPE
+    column.csv_unsafe = shcore::str_iendswith(data_type, "binary", "blob") ||
+                        mysqlshdk::db::Type::Bit == column.type ||
+                        mysqlshdk::db::Type::Geometry == column.type;
+    const auto extra = row->get_string(7, "");  // EXTRA
+    column.generated = extra.find(" GENERATED") != std::string::npos;
+    column.auto_increment = extra.find("auto_increment") != std::string::npos;
+    column.nullable = shcore::str_caseeq(row->get_string(5),
+                                         "YES");  // IS_NULLABLE
+
+    return column;
+  };
+
   iterate_tables_and_views(
       info,
-      [&table_columns](const std::string &schema_name,
-                       const std::string &table_name, Instance_cache::Table *,
-                       const mysqlshdk::db::IRow *row) {
-        Instance_cache::Column column;
-        // these can be NULL in 8.0, as per output of 'SHOW COLUMNS', but it's
-        // not likely, as they're NOT NULL in definition of mysql.columns hidden
-        // table
-        column.name = row->get_string(2, "");  // COLUMN_NAME
-        column.quoted_name = shcore::quote_identifier(column.name);
-        const auto data_type = row->get_string(3, "");  // DATA_TYPE
-        column.type = mysqlshdk::db::dbstring_to_type(
-            data_type, row->get_string(6));  // COLUMN_TYPE
-        column.csv_unsafe =
-            shcore::str_iendswith(data_type, "binary", "blob") ||
-            mysqlshdk::db::Type::Bit == column.type ||
-            mysqlshdk::db::Type::Geometry == column.type;
-        const auto extra = row->get_string(7, "");  // EXTRA
-        column.generated = extra.find(" GENERATED") != std::string::npos;
-        column.auto_increment =
-            extra.find("auto_increment") != std::string::npos;
-        column.nullable = shcore::str_caseeq(row->get_string(5),
-                                             "YES");  // IS_NULLABLE
-
+      [&table_columns, &create_column](
+          const std::string &schema_name, const std::string &table_name,
+          Instance_cache::Table *, const mysqlshdk::db::IRow *row) {
         table_columns[schema_name][table_name].emplace(
-            row->get_uint(4),
-            std::move(column));  // ORDINAL_POSITION
+            row->get_uint(4),  // ORDINAL_POSITION
+            create_column(row));
       },
-      [&view_columns](const std::string &schema_name,
-                      const std::string &view_name, Instance_cache::View *,
-                      const mysqlshdk::db::IRow *row) {
+      [&view_columns, &create_column](
+          const std::string &schema_name, const std::string &view_name,
+          Instance_cache::View *, const mysqlshdk::db::IRow *row) {
         view_columns[schema_name][view_name].emplace(
-            row->get_uint(4),
-            row->get_string(2, ""));  // ORDINAL_POSITION, COLUMN_NAME
+            row->get_uint(4),  // ORDINAL_POSITION
+            create_column(row));
       });
 
   for (auto &schema : table_columns) {
@@ -717,6 +726,10 @@ void Instance_cache_builder::fetch_columns() {
 
       for (auto &column : view.second) {
         v.all_columns.emplace_back(std::move(column.second));
+
+        if (!v.all_columns.back().generated) {
+          v.columns.emplace_back(&v.all_columns.back());
+        }
       }
     }
   }
@@ -920,7 +933,7 @@ void Instance_cache_builder::fetch_table_histograms() {
 }
 
 void Instance_cache_builder::fetch_table_partitions(
-    const Subobject_filters &partitions) {
+    const Partition_filters &partitions) {
   Profiler profiler{"fetching table partitions"};
 
   if (!has_tables()) {
@@ -1126,10 +1139,12 @@ void Instance_cache_builder::iterate_views(
   iterate_tables_and_views(info, {}, callback);
 }
 
-void Instance_cache_builder::set_schema_filter(const Filter &included,
-                                               const Filter &excluded) {
+void Instance_cache_builder::set_schema_filter() {
   m_schema_filter = "";
 
+  const auto &included = m_filters.schemas().included();
+  const auto &excluded = m_filters.schemas().excluded();
+
   if (!included.empty()) {
     m_schema_filter +=
         QH::case_sensitive_compare(k_schema_template, included, true);
@@ -1145,10 +1160,12 @@ void Instance_cache_builder::set_schema_filter(const Filter &included,
   }
 }
 
-void Instance_cache_builder::set_table_filter(const Object_filters &included,
-                                              const Object_filters &excluded) {
+void Instance_cache_builder::set_table_filter() {
   m_table_filter = "";
 
+  const auto &included = m_filters.tables().included();
+  const auto &excluded = m_filters.tables().excluded();
+
   if (!included.empty()) {
     m_table_filter += "(" +
                       QH::build_case_sensitive_filter(
@@ -1231,8 +1248,8 @@ std::string Instance_cache_builder::object_filter(
 }
 
 std::string Instance_cache_builder::trigger_filter(
-    const Iterate_table &info, const Subobject_filters &included,
-    const Subobject_filters &excluded) const {
+    const Iterate_table &info, const Trigger_filters &included,
+    const Trigger_filters &excluded) const {
   const auto filter_triggers = [&info](const std::string &schema,
                                        const Object_filters &tables) {
     const auto &trigger_column = info.extra_columns[0];
