@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -593,6 +593,11 @@ precedence:
 @li <b>aws_access_key_id</b>
 @li <b>aws_secret_access_key</b>
 @li <b>aws_session_token</b>
+-# Process specified by the <b>credential_process</b> setting from the
+<b>config</b> file for the specified profile:
+@li <b>AccessKeyId</b>
+@li <b>SecretAccessKey</b>
+@li <b>SessionToken</b>
 -# Settings from the <b>config</b> file for the specified profile:
 @li <b>aws_access_key_id</b>
 @li <b>aws_secret_access_key</b>
@@ -604,6 +609,22 @@ The items specified above correspond to the following credentials:
 @li the secret key associated with the AWS access key
 @li the AWS session token for the temporary security credentials
 
+The process/command line specified by the <b>credential_process</b> setting must
+write a JSON object to the standard output in the following form:
+
+@code
+{
+  "Version": 1,
+  "AccessKeyId": "AWS access key",
+  "SecretAccessKey": "secret key associated with the AWS access key",
+  "SessionToken": "temporary AWS session token, optional",
+  "Expiration": "RFC3339 timestamp, optional"
+}
+@endcode
+
+The <b>Expiration</b> key, if given, specifies when the credentials are going to
+expire, they will be automatically refreshed before this happens.
+
 The following credential handling rules apply:
 @li If the <b>s3Profile</b> option is set to a non-empty string, the environment
 variables are not used as a potential credential provider.
@@ -727,9 +748,9 @@ M - for Megabytes (n * 1'000'000 bytes), G - for Gigabytes (n * 1'000'000'000
 bytes), maxRate="2k" - limit to 2 kilobytes per second.
 @li <b>showProgress</b>: bool (default: true if stdout is a tty, false
 otherwise) - Enable or disable import progress information.
-@li <b>skipRows</b>: int (default: 0) - Skip first n rows of the data in the
-file. You can use this option to skip an initial header line containing column
-names.
+@li <b>skipRows</b>: int (default: 0) - Skip first N physical lines from each of
+the imported files. You can use this option to skip an initial header line
+containing column names.
 @li <b>dialect</b>: enum (default: "default") - Setup fields and lines options
 that matches specific data file format. Can be used as base dialect and
 customized with fieldsTerminatedBy, fieldsEnclosedBy, fieldsOptionallyEnclosed,
@@ -1073,8 +1094,16 @@ specified triggers from the dump. Strings are in format <b>schema</b>.<b>table</
 specified users from the dump. Each user is in the format of
 'user_name'[@'host']. If the host is not specified, all the accounts with the
 given user name are excluded.
+@li <b>handleGrantErrors</b>: "abort", "drop_account", "ignore" (default: abort)
+- Specifies action to be performed in case of errors related to the GRANT/REVOKE
+statements, "abort": throws an error and aborts the load, "drop_account":
+deletes the problematic account and continues, "ignore": ignores the error and
+continues loading the account.
 @li <b>ignoreExistingObjects</b>: bool (default false) - Load the dump even if
-it contains objects that already exist in the target database.
+it contains user accounts or DDL objects that already exist in the target
+database. If this option is set to false, any existing object results in an
+error. Setting it to true ignores existing objects, but the CREATE statements
+are still going to be executed.
 @li <b>ignoreVersion</b>: bool (default false) - Load the dump even if the
 major version number of the server where it was created is different from where
 it will be loaded.
@@ -1105,8 +1134,8 @@ given user name are included. By default, all users are included.
 @li <b>loadDdl</b>: bool (default: true) - Executes DDL/SQL scripts in the
 dump.
 @li <b>loadIndexes</b>: bool (default: true) - use together with
-‘deferTableIndexes’ to control whether secondary indexes should be recreated
-at the end of the load. Useful when loading DDL and data separately.
+<b>deferTableIndexes</b> to control whether secondary indexes should be
+recreated at the end of the load. Useful when loading DDL and data separately.
 @li <b>loadUsers</b>: bool (default: false) - Executes SQL scripts for user
 accounts, roles and grants contained in the dump. Note: statements for the
 current user will be skipped.
@@ -1315,6 +1344,10 @@ Primary Keys. Dumps created with this value cannot be used in MySQL Database
 Service instance with High Availability. Mutually exclusive with the
 <b>create_invisible_pks</b> value.
 
+<b>ignore_wildcard_grants</b> - Ignore errors from grants on schemas with
+wildcards, which are interpreted differently in systems where
+<b>partial_revokes</b> system variable is enabled.
+
 <b>skip_invalid_accounts</b> - Skips accounts which do not have a password or
 use authentication methods (plugins) not supported by the MySQL Database Service.
 
@@ -1332,6 +1365,10 @@ you will need to manually modify the schema before loading it.
 
 Please refer to the MySQL manual for details about DEFINER and SQL SECURITY.
 
+<b>strip_invalid_grants</b> - Strips grant statements which would fail when
+users are loaded, i.e. grants referring to a specific routine which does not
+exist.
+
 <b>strip_restricted_grants</b> - Certain privileges are restricted in the MySQL
 Database Service. Attempting to create users granting these privileges would
 fail, so this option allows dumped GRANT statements to be stripped of these
@@ -1473,8 +1510,8 @@ MySQL Database Service (MDS)
 @li <b>compatibility</b>: list of strings (default: empty) - Apply MySQL
 Database Service compatibility modifications when writing dump files. Supported
 values: "create_invisible_pks", "force_innodb", "ignore_missing_pks",
-"skip_invalid_accounts", "strip_definers", "strip_restricted_grants",
-"strip_tablespaces".)*");
+"ignore_wildcard_grants", "skip_invalid_accounts", "strip_definers",
+"strip_invalid_grants", "strip_restricted_grants", "strip_tablespaces".)*");
 
 REGISTER_HELP_DETAIL_TEXT(TOPIC_UTIL_DUMP_SCHEMAS_COMMON_OPTIONS, R"*(
 @li <b>excludeTables</b>: list of strings (default: empty) - List of tables or
