@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2022, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -75,6 +75,10 @@ class Signer {
   virtual bool refresh_auth_data() = 0;
 
   virtual bool auth_data_expired(time_t now) const = 0;
+
+  virtual bool is_authorization_error(const Response &response) const {
+    return Response::Status_code::UNAUTHORIZED == response.status;
+  }
 };
 
 class Signed_rest_service_config {
