@@ -185,8 +185,9 @@ PyTypeObject PyShFuncObjectType = {
 #endif
 #if PY_VERSION_HEX >= 0x03080000
     ,
-    0,  // tp_vectorcall
+    0  // tp_vectorcall
 #if PY_VERSION_HEX < 0x03090000
+    ,
     0  // tp_print
 #endif
 #endif
