@@ -10,8 +10,7 @@ function print_metadata_instance_addresses(session) {
 
 function get_recovery_user(session) {
     var res = session.runSql(
-        "SELECT user_name FROM mysql.slave_master_info " +
-        "WHERE Channel_name = 'group_replication_recovery'");
+        "SELECT user_name FROM mysql.slave_master_info WHERE Channel_name = 'group_replication_recovery'");
     var row = res.fetchOne();
     return row[0];
 }
@@ -25,8 +24,15 @@ function number_of_non_expiring_pwd_accounts(session, user) {
     return row[0];
 }
 
+function check_auto_increment_multi_primary(session, base_value) {
 
+    EXPECT_EQ(base_value, get_sysvar(session, "auto_increment_increment"), "Incorrect auto_increment_increment value.");
 
+    var server_id = session.runSql("SELECT @@server_id").fetchOne()[0];
+    var expected_offset = 1 + (server_id % base_value)
+
+    EXPECT_EQ(expected_offset, get_sysvar(session, "auto_increment_offset"), "Incorrect auto_increment_offset value.");
+}
 
 // BUG#27084767: CREATECLUSTER()/ ADDINSTANCE() DOES NOT CORRECTLY SET AUTO_INCREMENT_OFFSET
 //
@@ -48,7 +54,9 @@ function number_of_non_expiring_pwd_accounts(session, user) {
 
 //@ BUG#27084767: Initialize new instances
 testutil.deploySandbox(__mysql_sandbox_port1, "root", {report_host: hostname});
+testutil.snapshotSandboxConf(__mysql_sandbox_port1);
 testutil.deploySandbox(__mysql_sandbox_port2, "root", {report_host: hostname});
+testutil.snapshotSandboxConf(__mysql_sandbox_port2);
 
 shell.connect(__sandbox_uri1);
 
@@ -56,6 +64,7 @@ shell.connect(__sandbox_uri1);
 var session2 = mysql.getSession(__sandbox_uri2);
 session.runSql("SET GLOBAL offline_mode=1");
 session2.runSql("SET GLOBAL offline_mode=1");
+session2.close();
 
 //@ BUG#27084767: Create a cluster in single-primary mode
 var c = dba.createCluster('test', {gtidSetIsComplete: true});
@@ -76,6 +85,21 @@ EXPECT_EQ(2, get_sysvar(__mysql_sandbox_port2, "auto_increment_offset"));
 EXPECT_EQ(0, get_sysvar(__mysql_sandbox_port1, "offline_mode"));
 EXPECT_EQ(0, get_sysvar(__mysql_sandbox_port2, "offline_mode"));
 
+// ensure offline_mode wasn't persisted (BUG#34778797) {VER(<8.0.11)}
+dba.configureLocalInstance(__sandbox_uri1, {mycnfPath: testutil.getSandboxConfPath(__mysql_sandbox_port1)});
+dba.configureLocalInstance(__sandbox_uri2, {mycnfPath: testutil.getSandboxConfPath(__mysql_sandbox_port2)});
+EXPECT_THROWS(function() { testutil.getSandboxConf(__mysql_sandbox_port1, "offline_mode"); }, "Option 'offline_mode' does not exist in group 'mysqld'");
+EXPECT_THROWS(function() { testutil.getSandboxConf(__mysql_sandbox_port2, "offline_mode"); }, "Option 'offline_mode' does not exist in group 'mysqld'");
+
+//@<> ensure offline_mode wasn't persisted (BUG#34778797) {VER(>=8.0.11)}
+shell.connect(__sandbox_uri1);
+session2 = mysql.getSession(__sandbox_uri2);
+
+EXPECT_EQ(0, session.runSql("SELECT count(*) FROM performance_schema.persisted_variables WHERE (variable_name = 'offline_mode')").fetchOne()[0], `Variable 'offline_mode' is persisted in '${__sandbox_uri1}'`);
+EXPECT_EQ(0, session2.runSql("SELECT count(*) FROM performance_schema.persisted_variables WHERE (variable_name = 'offline_mode')").fetchOne()[0], `Variable 'offline_mode' is persisted in '${__sandbox_uri2}'`);
+
+session2.close();
+
 // Test in multi-primary mode
 
 //@ BUG#27084767: Dissolve the cluster
@@ -92,16 +116,8 @@ var c = dba.createCluster('test', {multiPrimary: true, force: true, clearReadOnl
 session.close();
 shell.connect(__sandbox_uri1);
 
-// Get the server_id to calculate the expected value of auto_increment_offset
-var result = session.runSql("SELECT @@server_id");
-var row = result.fetchOne();
-var server_id = row[0];
-
-var __expected_auto_inc_offset = 1 + server_id%7
-
 //@<> BUG#27084767: Verify the values of auto_increment_% in the seed instance in multi-primary mode
-EXPECT_EQ(7, get_sysvar(session, "auto_increment_increment"));
-EXPECT_EQ(__expected_auto_inc_offset, get_sysvar(session, "auto_increment_offset"));
+check_auto_increment_multi_primary(session, 7);
 
 //@ BUG#27084767: Add instance to cluster in multi-primary mode
 c.addInstance(__sandbox_uri2)
@@ -111,16 +127,30 @@ testutil.waitMemberState(__mysql_sandbox_port2, "ONLINE");
 session.close();
 shell.connect(__sandbox_uri2);
 
-// Get the server_id to calculate the expected value of auto_increment_offset
-var result = session.runSql("SELECT @@server_id");
-var row = result.fetchOne();
-var server_id = row[0];
-
-var __expected_auto_inc_offset = 1 + server_id%7
-
 //@<> BUG#27084767: Verify the values of auto_increment_% multi-primary
-EXPECT_EQ(7, get_sysvar(session, "auto_increment_increment"));
-EXPECT_EQ(__expected_auto_inc_offset, get_sysvar(session, "auto_increment_offset"));
+check_auto_increment_multi_primary(session, 7);
+
+//@<> BUG#34711038: Verify the values of auto_increment_% multi-primary
+shell.connect(__sandbox_uri1);
+
+c.removeInstance(__sandbox_uri2);
+
+var result = session.runSql(`SELECT cluster_id, address, instance_name, addresses, attributes, description FROM mysql_innodb_cluster_metadata.instances WHERE (address = '${hostname}:${__mysql_sandbox_port1}')`);
+var row = result.fetchOne();
+session.runSql("INSERT INTO mysql_innodb_cluster_metadata.instances(cluster_id, address, mysql_server_uuid, instance_name, addresses, attributes, description) VALUES (?, ?, ?, ?, ?, ?, ?)", [row[0], row[1], "aaaaaaaa-bbbb-cccc-dddd-aaaaaaaaaaaa", row[2], row[3], row[4], row[5]]);
+session.runSql("INSERT INTO mysql_innodb_cluster_metadata.instances(cluster_id, address, mysql_server_uuid, instance_name, addresses, attributes, description) VALUES (?, ?, ?, ?, ?, ?, ?)", [row[0], row[1], "aaaaaaaa-bbbb-cccc-dddd-bbbbbbbbbbbb", row[2], row[3], row[4], row[5]]);
+session.runSql("INSERT INTO mysql_innodb_cluster_metadata.instances(cluster_id, address, mysql_server_uuid, instance_name, addresses, attributes, description) VALUES (?, ?, ?, ?, ?, ?, ?)", [row[0], row[1], "aaaaaaaa-bbbb-cccc-dddd-cccccccccccc", row[2], row[3], row[4], row[5]]);
+session.runSql("INSERT INTO mysql_innodb_cluster_metadata.instances(cluster_id, address, mysql_server_uuid, instance_name, addresses, attributes, description) VALUES (?, ?, ?, ?, ?, ?, ?)", [row[0], row[1], "aaaaaaaa-bbbb-cccc-dddd-dddddddddddd", row[2], row[3], row[4], row[5]]);
+session.runSql("INSERT INTO mysql_innodb_cluster_metadata.instances(cluster_id, address, mysql_server_uuid, instance_name, addresses, attributes, description) VALUES (?, ?, ?, ?, ?, ?, ?)", [row[0], row[1], "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee", row[2], row[3], row[4], row[5]]);
+session.runSql("INSERT INTO mysql_innodb_cluster_metadata.instances(cluster_id, address, mysql_server_uuid, instance_name, addresses, attributes, description) VALUES (?, ?, ?, ?, ?, ?, ?)", [row[0], row[1], "aaaaaaaa-bbbb-cccc-dddd-ffffffffffff", row[2], row[3], row[4], row[5]]);
+
+c.addInstance(__sandbox_uri2);
+
+shell.connect(__sandbox_uri2);
+check_auto_increment_multi_primary(session, 8);
+
+shell.connect(__sandbox_uri1);
+check_auto_increment_multi_primary(session, 8);
 
 //@ BUG#27084767: Finalization
 c.disconnect()
@@ -220,12 +250,10 @@ testutil.destroySandbox(__mysql_sandbox_port2);
 WIPE_SHELL_LOG();
 testutil.deploySandbox(__mysql_sandbox_port1, "root", {"mysqlx":"0", report_host: hostname});
 testutil.deploySandbox(__mysql_sandbox_port2, "root", {"mysqlx":"0", report_host: hostname});
-testutil.deploySandbox(__mysql_sandbox_port3, "root", {report_host: hostname});
 
 shell.connect(__sandbox_uri1);
 c = dba.createCluster('noxplugin', {gtidSetIsComplete: true});
 c.addInstance(__sandbox_uri2);
-c.addInstance(__sandbox_uri3);
 
 var msg1 = "The X plugin is not enabled on instance '" + hostname + ":" + __mysql_sandbox_port1 + "'. No value will be assumed for the X protocol address.";
 var msg2 = "The X plugin is not enabled on instance '" + hostname + ":" + __mysql_sandbox_port2 + "'. No value will be assumed for the X protocol address.";
@@ -240,7 +268,6 @@ c.disconnect();
 session.close();
 testutil.destroySandbox(__mysql_sandbox_port1);
 testutil.destroySandbox(__mysql_sandbox_port2);
-testutil.destroySandbox(__mysql_sandbox_port3);
 
 // BUG28056944: cannot add instance after removing its metadata
 //@ BUG28056944 deploy sandboxes.
@@ -491,9 +518,11 @@ testutil.changeSandboxConf(__mysql_sandbox_port2, "foo", "bar");
 shell.options["dba.restartWaitTimeout"] = 1;
 c.addInstance(__sandbox_uri2, {interactive:true, recoveryMethod:"clone"});
 shell.options["dba.restartWaitTimeout"] = 60;
+testutil.waitSandboxDead(__mysql_sandbox_port2);
 
 //@<> BUG#30281908: restart the instance manually {VER(>= 8.0.17)}
 testutil.removeFromSandboxConf(__mysql_sandbox_port2, "foo");
+testutil.killSandbox(__mysql_sandbox_port2); //the server could still be restarting
 testutil.startSandbox(__mysql_sandbox_port2);
 testutil.waitMemberState(__mysql_sandbox_port2, "ONLINE");
 
