@@ -252,7 +252,8 @@
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": null
                 },
                 "status": "INVALIDATED",
                 "transactionSetConsistencyStatus": "OK"
@@ -301,7 +302,8 @@
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": null
                 },
                 "status": "INVALIDATED",
                 "transactionSetConsistencyStatus": "OK"
@@ -359,7 +361,8 @@
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             },
@@ -372,7 +375,8 @@
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             },
