@@ -54,7 +54,9 @@ NOTE: Some configuration options need to be fixed:
 ?{}
 
 Some variables need to be changed, but cannot be done dynamically on the server.
-Cluster admin user 'admin'@'%' created.
+Creating user admin@%.
+Account admin@% was successfully created.
+
 Configuring instance...
 The instance '<<<__address1>>>' was configured to be used in an InnoDB ReplicaSet.
 NOTE: MySQL server needs to be restarted for configuration changes to take effect.
@@ -72,7 +74,9 @@ applierWorkerThreads will be set to the default value of 4.
 ?{}
 
 The instance '<<<__address2>>>' is valid to be used in an InnoDB ReplicaSet.
-Cluster admin user 'admin'@'%' created.
+Creating user admin@%.
+Account admin@% was successfully created.
+
 The instance '<<<__address2>>>' is already ready to be used in an InnoDB ReplicaSet.
 ?{VER(>=8.0.23)}
 
@@ -87,6 +91,7 @@ A new replicaset with instance '<<<__address1>>>' will be created.
 This instance reports its own address as 127.0.0.1:<<<__mysql_sandbox_port1>>>
 <<<__address1>>>: Instance configuration is suitable.
 
+* Checking connectivity and SSL configuration...
 * Updating metadata...
 
 ReplicaSet object successfully created for <<<__address1>>>.
@@ -130,6 +135,8 @@ This instance reports its own address as 127.0.0.1:<<<__mysql_sandbox_port3>>>
 
 * Checking async replication topology...
 
+* Checking connectivity and SSL configuration...
+
 * Checking transaction state of the instance...
 
 NOTE: The target instance '<<<__address3>>>' has not been pre-provisioned (GTID set is empty). The Shell is unable to decide whether replication can completely recover its state.
@@ -153,26 +160,6 @@ The instance '<<<__address3>>>' was added to the replicaset and is replicating f
 //@<OUT> removeInstance
 The instance '127.0.0.1:<<<__mysql_sandbox_port2>>>' was removed from the replicaset.
 
-Adding instance to the replicaset...
-
-* Performing validation checks
-
-This instance reports its own address as 127.0.0.1:<<<__mysql_sandbox_port2>>>
-<<<__address2>>>: Instance configuration is suitable.
-
-* Checking async replication topology...
-
-* Checking transaction state of the instance...
-
-Incremental state recovery selected through the recoveryMethod option
-
-* Updating topology
-** Changing replication source of <<<__address2>>> to 127.0.0.1:<<<__mysql_sandbox_port1>>>
-** Waiting for new instance to synchronize with PRIMARY...
-
-
-The instance '<<<__address2>>>' was added to the replicaset and is replicating from 127.0.0.1:<<<__mysql_sandbox_port1>>>.
-
 //@<OUT> setPrimaryInstance
 127.0.0.1:<<<__mysql_sandbox_port3>>> will be promoted to PRIMARY of 'myrs'.
 The current PRIMARY is 127.0.0.1:<<<__mysql_sandbox_port1>>>.
@@ -206,7 +193,7 @@ The current PRIMARY is 127.0.0.1:<<<__mysql_sandbox_port1>>>.
 127.0.0.1:<<<__mysql_sandbox_port3>>> was promoted to PRIMARY.
 
 //@<OUT> forcePrimaryInstance (prepare)
-WARNING: Unable to connect to the PRIMARY of the replicaset myrs: MYSQLSH 51118: Could not open connection to '127.0.0.1:<<<__mysql_sandbox_port3>>>': Can't connect to MySQL server on '<<<libmysql_host_description('127.0.0.1', __mysql_sandbox_port3)>>>' ([[*]])
+WARNING: Unable to connect to the PRIMARY of the ReplicaSet myrs: MYSQLSH 51118: Could not open connection to '127.0.0.1:<<<__mysql_sandbox_port3>>>': Can't connect to MySQL server on '<<<libmysql_host_description('127.0.0.1', __mysql_sandbox_port3)>>>' ([[*]])
 Cluster change operations will not be possible unless the PRIMARY can be reached.
 If the PRIMARY is unavailable, you must either repair it or perform a forced failover.
 See \help forcePrimaryInstance for more information.
@@ -384,7 +371,8 @@ true
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": null
                 },
                 "status": "ONLINE"
             },
@@ -397,7 +385,8 @@ true
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": null
                 },
                 "status": "ONLINE"
             }
