@@ -31,7 +31,8 @@
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             }
@@ -77,6 +78,7 @@
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
                     "receiverTimeSinceLastMessage": "[[*]]",
                     "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3",
                     "source": "<<<hostname_ip>>>:<<<__mysql_sandbox_port1>>>"
                 },
                 "serverUuid": "5ef81566-9395-11e9-87e9-222222222222",
@@ -131,6 +133,7 @@
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
                     "receiverTimeSinceLastMessage": "[[*]]",
                     "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3",
                     "source": "<<<hostname_ip>>>:<<<__mysql_sandbox_port1>>>"
                 },
                 "serverUuid": "5ef81566-9395-11e9-87e9-222222222222",
@@ -168,7 +171,8 @@
                     },
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             }
@@ -221,6 +225,7 @@
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
                     "receiverTimeSinceLastMessage": "[[*]]",
                     "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3",
                     "source": "<<<hostname_ip>>>:<<<__mysql_sandbox_port1>>>"
                 },
                 "serverUuid": "5ef81566-9395-11e9-87e9-222222222222",
@@ -256,7 +261,8 @@
                     "applierWorkerThreads": 3,
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             }
@@ -309,7 +315,8 @@
                     "applierWorkerThreads": 4,
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             }
@@ -345,7 +352,8 @@
                     "applierThreadState": "<<<__replica_keyword_capital>>> has read all relay log; waiting for more updates",
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             }
@@ -387,6 +395,7 @@
                     "receiverStatus": "OFF",
                     "receiverThreadState": "",
                     "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3",
                     "source": "<<<hostname_ip>>>:<<<__mysql_sandbox_port1>>>"
                 },
                 "status": "OFFLINE",
@@ -419,7 +428,8 @@
                     "applierThreadState": <<<(__version_num<80023)?'"Slave has read all relay log; waiting for more updates",':'"Waiting for an event from Coordinator",\n                    "applierWorkerThreads": 4,'>>>
                     "receiverStatus": "ON",
                     "receiverThreadState": "[[*]]",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": null
                 },
                 "status": "ERROR"
             },
@@ -432,7 +442,8 @@
                     "applierThreadState": "<<<__replica_keyword_capital>>> has read all relay log; waiting for more updates",
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for <<<__source_keyword>>> to send event",
-                    "replicationLag": null
+                    "replicationLag": null,
+                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
                 },
                 "status": "ONLINE"
             }
@@ -447,7 +458,7 @@
 |                "status": "UNREACHABLE"|
 
 //@# Primary is down
-|WARNING: Unable to connect to the PRIMARY of the replicaset myrs: MYSQLSH 51118: Could not open connection to '<<<hostname_ip>>>:<<<__mysql_sandbox_port1>>>': Can't connect to MySQL server on '<<<libmysql_host_description(hostname_ip, __mysql_sandbox_port1)>>>'|
+|WARNING: Unable to connect to the PRIMARY of the ReplicaSet myrs: MYSQLSH 51118: Could not open connection to '<<<hostname_ip>>>:<<<__mysql_sandbox_port1>>>': Can't connect to MySQL server on '<<<libmysql_host_description(hostname_ip, __mysql_sandbox_port1)>>>'|
 |Cluster change operations will not be possible unless the PRIMARY can be reached.|
 |If the PRIMARY is unavailable, you must either repair it or perform a forced failover.|
 |See \help forcePrimaryInstance for more information.|
