@@ -33,6 +33,12 @@ The following operations are available at 'clusterset':
    set-primary-cluster
       Performs a safe switchover of the PRIMARY Cluster of the ClusterSet.
 
+   setup-admin-account
+      Create or upgrade an InnoDB ClusterSet admin account.
+
+   setup-router-account
+      Create or upgrade a MySQL account to use with MySQL Router.
+
    status
       Describe the status of the ClusterSet.
 
@@ -118,6 +124,10 @@ OPTIONS
             The Group Replication protocol stack to be used in the Cluster:
             XCom (legacy) or MySQL.
 
+--paxosSingleLeader=<bool>
+            Boolean value used to enable/disable the Group Communication engine
+            to operate with a single consensus leader.
+
 --recoveryMethod=<str>
             Preferred method for state recovery/provisioning. May be auto,
             clone or incremental. Default is auto.
