@@ -85,7 +85,7 @@ remove_local_progress_file()
 # WL14841-TSFR_3_3
 PREPARE_PAR_IS_SECRET_TEST()
 EXPECT_THROWS(lambda: util.load_dump(all_read_par_no_list, {"progressFile": local_progress_file}),
-  f"Error: Shell Error (54404): Util.load_dump: While 'Listing files': Could not access 'https://objectstorage.{config['region']}.oraclecloud.com/p/<secret>/n/{OS_NAMESPACE}/b/{OS_BUCKET_NAME}/o/': Either the bucket named '{OS_BUCKET_NAME}' does not exist in the namespace '{OS_NAMESPACE}' or you are not authorized to access it")
+  f"Error: Shell Error (54404): Util.load_dump: While 'Listing files': Could not access 'https://objectstorage.{config['region']}.oraclecloud.com/p/<secret>/n/{OS_NAMESPACE}/b/{OS_BUCKET_NAME}/o/': BucketNotFound: Either the bucket named '{OS_BUCKET_NAME}' does not exist in the namespace '{OS_NAMESPACE}' or you are not authorized to access it")
 EXPECT_PAR_IS_SECRET()
 
 #@<> WL14645-TSFR_1_10 - Failed load dump with prefix AnyObjectWrite PAR and ListObjects
