@@ -77,7 +77,7 @@ function get_socket_path(session, uri = undefined) {
 
   var row = session.runSql(`SELECT @@${"mysql" === shell.parseUri(uri).scheme ? "socket" : "mysqlx_socket"}, @@datadir`).fetchOne();
 
-  if (row[0][0] == '/')
+  if (row[0][0] == '/' || __os_type == 'windows')
     p = row[0];
   else if (row[1][row[1].length-1] == '/')
     p = row[1] + row[0];
@@ -116,12 +116,12 @@ function genlog_filter_reads(logs, pattern) {
       if (pattern === undefined) {
         if (sql.match("select 1") || sql.match(/select.*from.*/i) ||
             sql.match(/(show|begin|start transaction|rollback|.*@@hostname|.*@@server_uuid|select.*gtid)/i) ||
-            sql.match(/set (session|@)/i)) {
+            sql.match(/set (session|@)/i) || sql.match(/select\s*(service_get_write_locks|service_get_read_locks|service_release_locks)\(.*\)/i)) {
         } else {
           out.push(log);
         }
       } else {
-        if (sql.match(pattern) || sql.match("select 1")) {
+        if (sql.match(pattern) || sql.match("select 1") || sql.match(/select\s*(service_get_write_locks|service_get_read_locks|service_release_locks)\(.*\)/i)) {
         } else {
           out.push(log);
         }
@@ -222,7 +222,7 @@ function hasAwsEnvironment() {
 
 
 function hasAuthEnvironment(context) {
-  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO'].indexOf(context) == -1) {
+  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO', 'OCI_AUTH'].indexOf(context) == -1) {
     return false
   }
 
@@ -241,8 +241,7 @@ function hasAuthEnvironment(context) {
                  'LDAP_SASL_BIND_BASE_DN',
                  'LDAP_SASL_USER',
                  'LDAP_SASL_PWD',
-                 'LDAP_SASL_GROUP_SEARCH_FILTER',
-                 'MYSQL_PLUGIN_DIR'];
+                 'LDAP_SASL_GROUP_SEARCH_FILTER'];
   } else if (context == 'LDAP_KERBEROS') {
     variables = ['LDAP_KERBEROS_SERVER_HOST',
                  'LDAP_KERBEROS_SERVER_PORT',
@@ -254,6 +253,10 @@ function hasAuthEnvironment(context) {
   } else if (context == 'KERBEROS') {
     variables = ['KERBEROS_USER',
                  'KERBEROS_PWD'];
+  } else if (context == 'OCI_AUTH') {
+    variables = ['OCI_AUTH_URI',
+                 'OCI_AUTH_CONFIG_FILE',
+                 'OCI_AUTH_POSITIVE_TESTS'];
   }
 
   let missing=[];
@@ -271,6 +274,133 @@ function hasAuthEnvironment(context) {
 }
 
 
+function debugAuthPlugins() {
+  return os.getenv("MYSQLSH_DEBUG_AUTH_PLUGINS") || false;
+}
+
+
+function getAuthServerConfig(context) {
+  if (!isAuthMethodSupported(context)) {
+    return {};
+  }
+
+  const ext = __os_type == "windows" ? "dll" : "so";
+
+  if (context == 'LDAP_SIMPLE') {
+    var server_conf = {
+      "plugin-load-add": `authentication_ldap_simple.${ext}`,
+      "authentication_ldap_simple_server_host": `${LDAP_SIMPLE_SERVER_HOST}`,
+      "authentication_ldap_simple_server_port": parseInt(`${LDAP_SIMPLE_SERVER_PORT}`),
+      "authentication_ldap_simple_bind_base_dn": `${LDAP_SIMPLE_BIND_BASE_DN}`
+    };
+
+    if (__os_type == 'windows') {
+      server_conf['named_pipe'] = 1;
+      server_conf['socket'] = "MyNamedPipe";
+    }
+
+    return server_conf;
+  } else if (context == 'LDAP_SASL') {
+    return {
+      "plugin-load-add": `authentication_ldap_sasl.${ext}`,
+      "authentication_ldap_sasl_server_host": `${LDAP_SASL_SERVER_HOST}`,
+      "authentication_ldap_sasl_server_port": parseInt(`${LDAP_SASL_SERVER_PORT}`),
+      "authentication_ldap_sasl_group_search_filter": `${LDAP_SASL_GROUP_SEARCH_FILTER}`,
+      "authentication_ldap_sasl_bind_base_dn": `${LDAP_SASL_BIND_BASE_DN}`,
+      "authentication_ldap_sasl_log_status": 5,
+      "log_error_verbosity": 3
+    };
+  } else if (context == 'LDAP_KERBEROS') {
+    return {
+      "plugin-load-add": `authentication_ldap_sasl.${ext}`,
+      "authentication_ldap_sasl_server_host": `${LDAP_KERBEROS_SERVER_HOST}`,
+      "authentication_ldap_sasl_server_port": parseInt(`${LDAP_KERBEROS_SERVER_PORT}`),
+      "authentication_ldap_sasl_bind_base_dn": `${LDAP_KERBEROS_BIND_BASE_DN}`,
+      "authentication_ldap_sasl_user_search_attr": `${LDAP_KERBEROS_USER_SEARCH_ATTR}`,
+      "authentication_ldap_sasl_bind_root_dn": `${LDAP_KERBEROS_BIND_ROOT_DN}`,
+      "authentication_ldap_sasl_bind_root_pwd": `${LDAP_KERBEROS_BIND_ROOT_PWD}`,
+      "authentication_ldap_sasl_group_search_filter": `${LDAP_KERBEROS_GROUP_SEARCH_FILTER}`,
+      "authentication_ldap_sasl_auth_method_name": 'GSSAPI',
+      "authentication_ldap_sasl_log_status": 5,
+      "log_error_verbosity": 3,
+      "net_read_timeout": 360,
+      "connect_timeout": 360
+    };
+  } else if (context == 'KERBEROS') {
+    const keytab_file = os.path.join(__tmp_dir, "mysql2.keytab");
+    testutil.cpfile(os.path.join(__data_path, "keytab", "mysql.keytab"), keytab_file);
+
+    return {
+      "plugin-load-add": `authentication_kerberos.${ext}`,
+      "authentication_kerberos_service_principal": "mysql_service/kerberos_auth_host@MYSQL.LOCAL",
+      "authentication_kerberos_service_key_tab": keytab_file,
+      "net_read_timeout": 360,
+      "connect_timeout": 360,
+      "log_error_verbosity": 3
+    };
+  } else if (context == 'FIDO') {
+    return {
+      "plugin-load-add": `authentication_fido.${ext}`
+    };
+  }
+
+  return {};
+}
+
+
+function isAuthMethodSupported(context) {
+  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO', 'OCI_AUTH'].indexOf(context) == -1) {
+    return false;
+  }
+
+  var plugin = '';
+
+  if (context == 'LDAP_SIMPLE') {
+    plugin = 'authentication_ldap_simple';
+  } else if (context == 'LDAP_SASL') {
+    plugin = 'authentication_ldap_sasl';
+  } else if (context == 'LDAP_KERBEROS') {
+    plugin = 'authentication_ldap_sasl';
+  } else if (context == 'KERBEROS') {
+    plugin = 'authentication_kerberos';
+  } else if (context == 'FIDO') {
+    plugin = 'authentication_fido';
+  } else if (context == 'OCI_AUTH') {
+    // we're using an external server, so this is always supported
+    return true;
+  }
+
+  var plugin_supported = true;
+  const s = shell.openSession(__mysqluripwd);
+
+  try {
+    ensure_plugin_enabled(plugin, s);
+  } catch (error) {
+    plugin_supported = false;
+  }
+
+  if (plugin_supported) {
+    if (context == 'LDAP_KERBEROS') {
+      try {
+        s.runSql("SET @@GLOBAL.authentication_ldap_sasl_auth_method_name = 'GSSAPI'");
+      } catch (error) {
+        plugin_supported = false;
+      }
+    }
+
+    try {
+      ensure_plugin_disabled(plugin, s);
+    } catch (error) {
+      // ignore any errors
+    }
+  }
+
+  s.close();
+
+  return plugin_supported;
+}
+
+
 function set_sysvar(session, variable, value) {
   session.runSql("SET GLOBAL " + variable + " = ?", [value]);
 }
@@ -1719,3 +1849,27 @@ function clone_installed(session) {
         return row[0] == "ACTIVE";
     return false;
 }
+
+
+function snapshot_tables(session, tables) {
+  snapshot = {};
+  for (tbl of tables) {
+    res = session.runSql(`select * from ${tbl} use index (primary)`).fetchAll();
+    snapshot[tbl] = res;
+  }
+  return snapshot;
+}
+
+function CHECK_TABLE_SNAPSHOTS(session, snapshot) {
+  for (tbl in snapshot) {
+    before = snapshot[tbl];
+    after = session.runSql(`select * from ${tbl} use index (primary)`).fetchAll();
+    if (before.length == after.length) {
+      for (i in before) {
+        EXPECT_EQ(before[i], after[i], `${tbl}:${i}`)
+      }
+    } else {
+      EXPECT_EQ(before, after, tbl);
+    }
+  }
+}
